@@ -63,15 +63,16 @@ View the results using::
 from __future__ import absolute_import
 from argparse import RawDescriptionHelpFormatter, ArgumentParser
 import os
-import time
-import glob
-from itertools import chain
+import sys
+sys.path.append('.')
+import csv
 
 import numpy as nm
 import matplotlib.pyplot as plt
 
 from sfepy.base.base import output, Struct
 from sfepy.base.ioutils import ensure_path, remove_files_patterns, save_options
+from sfepy.base.timing import Timer
 from sfepy.discrete.fem import Mesh, FEDomain, Field
 from sfepy.discrete.common.region import Region
 from sfepy.discrete import (FieldVariable, Material, Integral, Function,
@@ -188,8 +189,14 @@ def solve_problem(mesh_filename, options, comm):
 
     output('rank', rank, 'of', size)
 
-    mesh = Mesh.from_file(mesh_filename)
+    stats = Struct()
+    timer = Timer('solve_timer')
 
+    timer.start()
+    mesh = Mesh.from_file(mesh_filename)
+    stats.t_read_mesh = timer.stop()
+
+    timer.start()
     if rank == 0:
         cell_tasks = pl.partition_mesh(mesh, size, use_metis=options.metis,
                                        verbose=True)
@@ -197,15 +204,20 @@ def solve_problem(mesh_filename, options, comm):
     else:
         cell_tasks = None
 
+    stats.t_partition_mesh = timer.stop()
+
     output('creating global domain and field...')
-    tt = time.clock()
+    timer.start()
+
     domain = FEDomain('domain', mesh)
     omega = domain.create_region('Omega', 'all')
     field = Field.from_args('fu', nm.float64, 1, omega, approx_order=order)
-    output('...done in', time.clock() - tt)
+
+    stats.t_create_global_fields = timer.stop()
+    output('...done in', timer.dt)
 
     output('distributing field %s...' % field.name)
-    tt = time.clock()
+    timer.start()
 
     distribute = pl.distribute_fields_dofs
     lfds, gfds = distribute([field], cell_tasks,
@@ -215,7 +227,8 @@ def solve_problem(mesh_filename, options, comm):
                             comm=comm, verbose=True)
     lfd = lfds[0]
 
-    output('...done in', time.clock() - tt)
+    stats.t_distribute_fields_dofs = timer.stop()
+    output('...done in', timer.dt)
 
     if rank == 0:
         dof_maps = gfds[0].dof_maps
@@ -230,7 +243,7 @@ def solve_problem(mesh_filename, options, comm):
                                   options.output_dir, size)
 
     output('creating local problem...')
-    tt = time.clock()
+    timer.start()
 
     omega_gi = Region.from_cells(lfd.cells, field.domain)
     omega_gi.finalize()
@@ -238,20 +251,21 @@ def solve_problem(mesh_filename, options, comm):
 
     pb = create_local_problem(omega_gi, order)
 
-    output('...done in', time.clock() - tt)
-
     variables = pb.get_variables()
     eqs = pb.equations
 
     u_i = variables['u_i']
     field_i = u_i.field
 
+    stats.t_create_local_problem = timer.stop()
+    output('...done in', timer.dt)
+
     if options.plot:
         ppd.plot_local_dofs([None, None], field, field_i, omega_gi,
                             options.output_dir, rank)
 
     output('allocating global system...')
-    tt = time.clock()
+    timer.start()
 
     sizes, drange = pl.get_sizes(lfd.petsc_dofs_range, field.n_nod, 1)
     output('sizes:', sizes)
@@ -265,10 +279,11 @@ def solve_problem(mesh_filename, options, comm):
                                               is_overlap=True, comm=comm,
                                               verbose=True)
 
-    output('...done in', time.clock() - tt)
+    stats.t_allocate_global_system = timer.stop()
+    output('...done in', timer.dt)
 
     output('evaluating local problem...')
-    tt = time.clock()
+    timer.start()
 
     state = State(variables)
     state.fill(0.0)
@@ -278,10 +293,11 @@ def solve_problem(mesh_filename, options, comm):
     # This must be after pl.create_petsc_system() call!
     mtx_i = eqs.eval_tangent_matrices(state(), pb.mtx_a)
 
-    output('...done in', time.clock() - tt)
+    stats.t_evaluate_local_problem = timer.stop()
+    output('...done in', timer.dt)
 
     output('assembling global system...')
-    tt = time.clock()
+    timer.start()
 
     apply_ebc_to_matrix(mtx_i, u_i.eq_map.eq_ebc)
     pl.assemble_rhs_to_petsc(prhs, rhs_i, pdofs, drange, is_overlap=True,
@@ -289,20 +305,22 @@ def solve_problem(mesh_filename, options, comm):
     pl.assemble_mtx_to_petsc(pmtx, mtx_i, pdofs, drange, is_overlap=True,
                              comm=comm, verbose=True)
 
-    output('...done in', time.clock() - tt)
+    stats.t_assemble_global_system = timer.stop()
+    output('...done in', timer.dt)
 
     output('creating solver...')
-    tt = time.clock()
+    timer.start()
 
     conf = Struct(method='cg', precond='gamg', sub_precond='none',
                   i_max=10000, eps_a=1e-50, eps_r=1e-5, eps_d=1e4, verbose=True)
     status = {}
     ls = PETScKrylovSolver(conf, comm=comm, mtx=pmtx, status=status)
 
-    output('...done in', time.clock() - tt)
+    stats.t_create_solver = timer.stop()
+    output('...done in', timer.dt)
 
     output('solving...')
-    tt = time.clock()
+    timer.start()
 
     psol = ls(prhs, psol)
 
@@ -316,10 +334,11 @@ def solve_problem(mesh_filename, options, comm):
 
     gather(psol, psol_i)
 
-    output('...done in', time.clock() - tt)
+    stats.t_solve = timer.stop()
+    output('...done in', timer.dt)
 
     output('saving solution...')
-    tt = time.clock()
+    timer.start()
 
     u_i.set_data(sol0_i)
     out = u_i.create_output()
@@ -350,11 +369,44 @@ def solve_problem(mesh_filename, options, comm):
 
             out['u'].mesh.write(filename, io='auto', out=out)
 
-    output('...done in', time.clock() - tt)
+    stats.t_save_solution = timer.stop()
+    output('...done in', timer.dt)
+
+    stats.t_total = timer.total
+
+    stats.n_dof = sizes[1]
+    stats.n_dof_local = sizes[0]
+    stats.n_cell = omega.shape.n_cell
+    stats.n_cell_local = omega_gi.shape.n_cell
 
     if options.show:
         plt.show()
 
+    return stats
+
+def save_stats(filename, pars, stats, overwrite, rank, comm=None):
+    out = stats.to_dict()
+    names = sorted(out.keys())
+    shape_dict = {'n%d' % ii : pars.shape[ii] for ii in range(pars.dim)}
+    keys = ['size', 'rank', 'dim'] + list(shape_dict.keys()) + ['order'] + names
+
+    out['size'] = comm.size
+    out['rank'] = rank
+    out['dim'] = pars.dim
+    out.update(shape_dict)
+    out['order'] = pars.order
+
+    if rank == 0 and overwrite:
+        with open(filename, 'w') as fd:
+            writer = csv.DictWriter(fd, fieldnames=keys)
+            writer.writeheader()
+            writer.writerow(out)
+
+    else:
+        with open(filename, 'a') as fd:
+            writer = csv.DictWriter(fd, fieldnames=keys)
+            writer.writerow(out)
+
 helps = {
     'output_dir' :
     'output directory',
@@ -383,6 +435,10 @@ helps = {
     'save inter-task regions for debugging partitioning problems',
     'show' :
     'show partitioning plots (implies --plot)',
+    'stats_filename' :
+    'name of the stats file for storing elapsed time statistics',
+    'new_stats' :
+    'create a new stats file with a header line (overwrites existing!)',
     'silent' : 'do not print messages to screen',
     'clear' :
     'clear old solution files from output directory'
@@ -426,6 +482,12 @@ def main():
     parser.add_argument('--save-inter-regions',
                         action='store_true', dest='save_inter_regions',
                         default=False, help=helps['save_inter_regions'])
+    parser.add_argument('--stats', metavar='filename',
+                        action='store', dest='stats_filename',
+                        default=None, help=helps['stats_filename'])
+    parser.add_argument('--new-stats',
+                        action='store_true', dest='new_stats',
+                        default=False, help=helps['new_stats'])
     parser.add_argument('--silent',
                         action='store_true', dest='silent',
                         default=False, help=helps['silent'])
@@ -453,6 +515,14 @@ def main():
 
     mesh_filename = os.path.join(options.output_dir, 'para.h5')
 
+    dim = 2 if options.is_2d else 3
+    dims = nm.array(eval(options.dims), dtype=nm.float64)[:dim]
+    shape = nm.array(eval(options.shape), dtype=nm.int32)[:dim]
+    centre = nm.array(eval(options.centre), dtype=nm.float64)[:dim]
+    output('dimensions:', dims)
+    output('shape:     ', shape)
+    output('centre:    ', centre)
+
     if comm.rank == 0:
         from sfepy.mesh.mesh_generators import gen_block_mesh
 
@@ -466,15 +536,6 @@ def main():
         save_options(os.path.join(output_dir, 'options.txt'),
                      [('options', vars(options))])
 
-        dim = 2 if options.is_2d else 3
-        dims = nm.array(eval(options.dims), dtype=nm.float64)[:dim]
-        shape = nm.array(eval(options.shape), dtype=nm.int32)[:dim]
-        centre = nm.array(eval(options.centre), dtype=nm.float64)[:dim]
-
-        output('dimensions:', dims)
-        output('shape:     ', shape)
-        output('centre:    ', centre)
-
         mesh = gen_block_mesh(dims, shape, centre, name='block-fem',
                               verbose=True)
         mesh.write(mesh_filename, io='auto')
@@ -483,7 +544,21 @@ def main():
 
     output('field order:', options.order)
 
-    solve_problem(mesh_filename, options, comm)
+    stats = solve_problem(mesh_filename, options, comm)
+    output(stats)
+
+    if options.stats_filename:
+        if comm.rank == 0:
+            ensure_path(options.stats_filename)
+        comm.barrier()
+
+        pars = Struct(dim=dim, shape=shape, order=options.order)
+        pl.call_in_rank_order(
+            lambda rank, comm:
+            save_stats(options.stats_filename, pars, stats, options.new_stats,
+                       rank, comm),
+            comm
+        )
 
 if __name__ == '__main__':
     main()
