@@ -145,6 +145,7 @@ options = {
     'output_dir': 'output',
     'coefs_filename': 'coefs_le_up',
     'recovery_hook': 'recovery_le',
+    'multiprocessing': False,
 }
 
 equation_corrs = {
@@ -193,15 +194,12 @@ requirements = {
         'set_variables': [('Pi', 'pis', 'u')],
         'class': cb.CorrDimDim,
         'save_name': 'corrs_le',
-        'dump_variables': ['u', 'p'],
         'is_linear': True,
     },
 }
 
 solvers = {
-    'ls': ('ls.scipy_iterative', {
-        'method': 'cg',
-    }),
+    'ls': ('ls.auto_iterative', {}),
     'newton': ('nls.newton', {
         'i_max': 1,
         'eps_a': 1e2,
