@@ -1,12 +1,14 @@
-import time
+from __future__ import absolute_import
+from __future__ import print_function
 import sys
 import os
 from copy import copy, deepcopy
-from types import UnboundMethodType
-from getch import getch
+from types import MethodType
+from .getch import getch
 
 import numpy as nm
 import scipy.sparse as sp
+import six
 
 real_types = [nm.float64]
 complex_types = [nm.complex128]
@@ -19,10 +21,11 @@ sfepy_config_dir = os.path.expanduser('~/.sfepy')
 if not os.path.exists(sfepy_config_dir):
     os.makedirs(sfepy_config_dir)
 
-if sys.version[0] < '3':
+if sys.version_info[0] < 3:
+    PY3 = False
     basestr = basestring
-
 else:
+    PY3 = True
     basestr = str
 
 def get_debug():
@@ -92,31 +95,32 @@ def get_debug():
 
 debug = get_debug()
 
-def mark_time(times, msg=None):
+def debug_on_error():
     """
-    Time measurement utility.
-
-    Measures times of execution between subsequent calls using
-    time.clock(). The time is printed if the msg argument is not None.
-
-    Examples
-    --------
-
-    >>> times = []
-    >>> mark_time(times)
-    ... do something
-    >>> mark_time(times, 'elapsed')
-    elapsed 0.1
-    ... do something else
-    >>> mark_time(times, 'elapsed again')
-    elapsed again 0.05
-    >>> times
-    [0.10000000000000001, 0.050000000000000003]
+    Start debugger at the line where an exception was raised.
     """
-    tt = time.clock()
-    times.append(tt)
-    if (msg is not None) and (len(times) > 1):
-        print msg, times[-1] - times[-2]
+    try:
+        from IPython.core import ultratb
+
+        except_hook = ultratb.FormattedTB(mode='Verbose',
+                                          color_scheme='Linux', call_pdb=1)
+
+    except ImportError:
+        def except_hook(etype, value, tb):
+            if hasattr(sys, 'ps1') or not sys.stderr.isatty():
+                # We are in interactive mode or we don't have a tty-like
+                # device, so we call the default hook.
+                sys.__excepthook__(etype, value, tb)
+
+            else:
+                import traceback, pdb
+                # We are NOT in interactive mode, print the exception...
+                traceback.print_exception(etype, value, tb)
+                print()
+                # ...then start the debugger in post-mortem mode.
+                pdb.post_mortem(tb)
+
+    sys.excepthook = except_hook
 
 def import_file(filename, package_name=None, can_reload=True):
     """
@@ -148,10 +152,15 @@ def import_file(filename, package_name=None, can_reload=True):
         mod = __import__(name)
 
     if (name in sys.modules) and can_reload:
-        reload(mod)
+        if PY3:
+            import importlib
+            importlib.reload(mod)
+
+        else:
+            reload(mod)
 
     if remove_path:
-        sys.path.pop(-1)
+        sys.path.remove(path)
 
     return mod
 
@@ -175,10 +184,10 @@ def try_imports(imports, fail_msg=None):
     msgs = []
     for imp in imports:
         try:
-            exec imp
+            exec(imp)
             break
 
-        except Exception, inst:
+        except Exception as inst:
             msgs.append(str(inst))
 
     else:
@@ -188,11 +197,27 @@ def try_imports(imports, fail_msg=None):
 
     return locals()
 
-def python_shell():
+def python_shell(frame=0):
     import code
-    frame = sys._getframe(1)
+    frame = sys._getframe(frame+1)
     code.interact(local=frame.f_locals)
 
+def ipython_shell(frame=0):
+    from IPython.terminal.embed import InteractiveShellEmbed
+    ipshell = InteractiveShellEmbed()
+
+    ipshell(stack_depth=frame+1)
+
+def shell(frame=0):
+    """
+    Embed an IPython (if available) or regular Python shell in the given frame.
+    """
+    try:
+        ipython_shell(frame=frame+2)
+
+    except ImportError:
+        python_shell(frame=frame+1)
+
 def assert_(condition, msg='assertion failed!'):
     if not condition:
         raise ValueError(msg)
@@ -212,11 +237,11 @@ def pause(msg=None):
     f = sys._getframe(1)
     ff = f.f_code
     if (msg):
-        print '%s, %d: %s(), %d: %s' % (ff.co_filename, ff.co_firstlineno,
-                                        ff.co_name, f.f_lineno, msg)
+        print('%s, %d: %s(), %d: %s' % (ff.co_filename, ff.co_firstlineno,
+                                        ff.co_name, f.f_lineno, msg))
     else:
-        print '%s, %d: %s(), %d' % (ff.co_filename, ff.co_firstlineno,
-                                    ff.co_name, f.f_lineno)
+        print('%s, %d: %s(), %d' % (ff.co_filename, ff.co_firstlineno,
+                                    ff.co_name, f.f_lineno))
     spause()
 
 ##
@@ -234,7 +259,7 @@ def spause(msg=None):
     This is useful for debugging. This function is called from pause().
     """
     if (msg):
-        print msg
+        print(msg)
     sys.stdout.flush()
     ch = getch()
     if ch == 'q':
@@ -250,7 +275,7 @@ class Struct(object):
             self.__dict__.update(kwargs)
 
     def _format_sequence(self, seq, threshold):
-        threshold_half = threshold / 2
+        threshold_half = threshold // 2
 
         if len(seq) > threshold:
             out = ', '.join(str(ii) for ii in seq[:threshold_half]) \
@@ -290,7 +315,7 @@ class Struct(object):
         ss += '\n'
 
         if keys is None:
-            keys = self.__dict__.keys()
+            keys = list(self.__dict__.keys())
 
         str_attrs = sorted(Struct.get(self, '_str_attrs', keys))
         printed_keys = []
@@ -322,7 +347,7 @@ class Struct(object):
                     ss += '  %s:\n%s\n' % (key, aux[1:])
 
             elif isinstance(val, dict):
-                sval = self._format_sequence(val.keys(), threshold)
+                sval = self._format_sequence(list(val.keys()), threshold)
                 sval = sval.replace('\n', '\n    ')
                 ss += '  %s:\n    dict with keys: %s\n' % (key, sval)
 
@@ -369,7 +394,7 @@ class Struct(object):
         attribute and its counterpart in other are both Structs - these are
         merged then."""
         new = copy(self)
-        for key, val in other.__dict__.iteritems():
+        for key, val in six.iteritems(other.__dict__):
             if hasattr(new, key):
                 sval = getattr(self, key)
                 if issubclass(sval.__class__, Struct) and \
@@ -387,7 +412,7 @@ class Struct(object):
         """Merge Structs in place. Attributes of self are left unchanged
         unless an attribute and its counterpart in other are both Structs -
         these are merged then."""
-        for key, val in other.__dict__.iteritems():
+        for key, val in six.iteritems(other.__dict__):
             if hasattr(self, key):
                 sval = getattr(self, key)
                 if issubclass(sval.__class__, Struct) and \
@@ -401,12 +426,12 @@ class Struct(object):
         """
         As __str__(), but for class attributes.
         """
-        return self._str(self.__class__.__dict__.keys())
+        return self._str(list(self.__class__.__dict__.keys()))
 
     # 08.03.2005, c
     def str_all(self):
         ss = "%s\n" % self.__class__
-        for key, val in self.__dict__.iteritems():
+        for key, val in six.iteritems(self.__dict__):
             if issubclass(self.__dict__[key].__class__, Struct):
                 ss += "  %s:\n" % key
                 aux = "\n" + self.__dict__[key].str_all()
@@ -522,7 +547,7 @@ class Container(Struct):
                 self._objs[ii] = obj
                 self.names[ii] = obj.name
 
-        except (IndexError, ValueError), msg:
+        except (IndexError, ValueError) as msg:
             raise IndexError(msg)
 
     def __getitem__(self, ii):
@@ -534,12 +559,31 @@ class Container(Struct):
 
             return  self._objs[ii]
 
-        except (IndexError, ValueError), msg:
+        except (IndexError, ValueError) as msg:
             raise IndexError(msg)
 
     def __iter__(self):
         return self._objs.__iter__()
 
+    def __add__(self, other):
+        """
+        Add items of `other` to `self`.
+        """
+        new = Container()
+        objs = self._objs + other._objs
+        new.update(objs)
+
+        return new
+
+    def __iadd__(self, other):
+        """
+        Add items of `other` to `self` in place.
+        """
+        self.extend(copy(other._objs))
+        self.update()
+
+        return self
+
     ##
     # 18.07.2006, c
     def __len__(self):
@@ -627,7 +671,7 @@ class Container(Struct):
     ##
     # 12.06.2007, c
     def print_names(self):
-        print [obj.name for obj in self._objs]
+        print([obj.name for obj in self._objs])
 
     def get_names(self):
         return [obj.name for obj in self._objs]
@@ -675,9 +719,9 @@ class OneTypeList(list):
             if ir:
                 return list.__getitem__(self, ir[0])
             else:
-                raise IndexError, ii
+                raise IndexError(ii)
         else:
-            raise IndexError, ii
+            raise IndexError(ii)
 
     def __str__(self):
         ss = "[\n"
@@ -700,7 +744,7 @@ class OneTypeList(list):
     ##
     # 12.06.2007, c
     def print_names(self):
-        print [ii.name for ii in self]
+        print([ii.name for ii in self])
 
     def get_names(self):
         return [ii.name for ii in self]
@@ -789,7 +833,7 @@ class Output(Struct):
             if msg.startswith('...'):
                 self.level -= 1
 
-            print self._prefix + ('  ' * self.level) + msg
+            print(self._prefix + ('  ' * self.level) + msg)
 
             if msg.endswith('...'):
                 self.level += 1
@@ -801,7 +845,7 @@ class Output(Struct):
             else:
                 fd = filename
 
-            print >>fd, self._prefix + ('  ' * self.level) + msg
+            print(self._prefix + ('  ' * self.level) + msg, file=fd)
 
             if isinstance(filename, basestr):
                 fd.close()
@@ -828,7 +872,7 @@ class Output(Struct):
             if msg.startswith('...'):
                 self.level -= 1
 
-            print self._prefix + ('  ' * self.level) + msg
+            print(self._prefix + ('  ' * self.level) + msg)
 
             print_to_file(filename, msg)
 
@@ -888,7 +932,7 @@ output = Output('sfepy:')
 
 def configure_output(options):
     """
-    Configure the standard :function:`output()` function using
+    Configure the standard :func:`output()` function using
     `output_log_name` and `output_screen` attributes of `options`.
 
     Parameters
@@ -907,17 +951,17 @@ def print_structs(objs):
     """Print Struct instances in a container, works recursively. Debugging
     utility function."""
     if isinstance(objs, dict):
-        for key, vals in objs.iteritems():
-            print key
+        for key, vals in six.iteritems(objs):
+            print(key)
             print_structs(vals)
     elif isinstance(objs, list):
         for vals in objs:
             print_structs(vals)
     else:
-        print objs
+        print(objs)
 
 def iter_dict_of_lists(dol, return_keys=False):
-    for key, vals in dol.iteritems():
+    for key, vals in six.iteritems(dol):
         for ii, val in enumerate(vals):
             if return_keys:
                 yield key, ii, val
@@ -954,7 +998,7 @@ def dict_to_struct(*args, **kwargs):
             else:
                 aux = {}
 
-            for key, val in arg.iteritems():
+            for key, val in six.iteritems(arg):
                 if type(val) == dict:
                     try:
                         flag[level + 1]
@@ -979,13 +1023,33 @@ def dict_to_struct(*args, **kwargs):
 
     return out
 
+def structify(obj):
+    """
+    Convert a (nested) dict `obj` into a (nested) Struct.
+    """
+    out = Struct(**obj)
+    for key, val in out.to_dict().items():
+        if isinstance(val, dict):
+            out.__dict__[key] = structify(val)
+    return out
+
+def is_string(var):
+    return isinstance(var, basestr)
+
+def is_integer(var):
+    if PY3:
+        return isinstance(var, int)
+
+    else:
+        return isinstance(var, (int, long))
+
 ##
 # 23.01.2006, c
 def is_sequence(var):
-    if issubclass(var.__class__, tuple) or issubclass(var.__class__, list):
-        return True
-    else:
+    from collections import Sequence
+    if isinstance(var, basestr):
         return False
+    return isinstance(var, Sequence)
 
 ##
 # 17.10.2007, c
@@ -1000,8 +1064,11 @@ def insert_static_method(cls, function):
 ##
 # 23.10.2007, c
 def insert_method(instance, function):
-    setattr(instance, function.__name__,
-             UnboundMethodType(function, instance, instance.__class__))
+    if PY3:
+        meth = MethodType(function, instance)
+    else:
+        meth = MethodType(function, instance, type(instance))
+    setattr(instance, function.__name__, meth)
 
 def use_method_with_name(instance, method, new_name):
     setattr(instance, new_name, method)
@@ -1020,7 +1087,7 @@ def find_subclasses(context, classes, omit_unnamed=False, name_attr='name'):
                                         TimeSteppingSolver, EigenvalueSolver,
                                         OptimizationSolver])
     """
-    var_dict = context.items()
+    var_dict = list(context.items())
     table = {}
 
     for key, var in var_dict:
@@ -1106,8 +1173,9 @@ def update_dict_recursively(dst, src, tuples_too=False,
 
             if tuples_too and isinstance(dst[key], tuple) \
                    and isinstance(src[key], tuple):
-                dst[key] = tuple(map(tuplezip,
-                                     zip(src[key], dst[key]))[:len(dst[key])])
+                out = map(tuplezip, zip(src[key], dst[key]))
+                out = tuple(out)
+                dst[key] = out[:len(dst[key])]
                 continue
 
         if overwrite_by_none or not src[key] is None:
@@ -1172,7 +1240,7 @@ def edit_dict_strings(str_dict, old, new, recur=False):
     """
     if isinstance(old, basestr):
         new_dict = {}
-        for key, val in str_dict.iteritems():
+        for key, val in six.iteritems(str_dict):
             if isinstance(val, basestr):
                 new_dict[key] = val.replace(old, new)
 
@@ -1214,7 +1282,7 @@ def invert_dict(d, is_val_tuple=False, unique=True):
     """
     di = {}
 
-    for key, val in d.iteritems():
+    for key, val in six.iteritems(d):
         if unique:
             if is_val_tuple:
                 for v in val:
@@ -1239,7 +1307,7 @@ def remap_dict(d, map):
     Utility function to remap state dict keys according to var_map.
     """
     out = {}
-    for new_key, key in map.iteritems():
+    for new_key, key in six.iteritems(map):
         out[new_key] = d[key]
 
     return out
@@ -1260,7 +1328,7 @@ def dict_from_keys_init(keys, seq_class=None):
 ##
 # 16.10.2006, c
 def dict_extend(d1, d2):
-    for key, val in d1.iteritems():
+    for key, val in six.iteritems(d1):
         val.extend(d2[key])
 
 def get_subdict(adict, keys):
@@ -1270,7 +1338,7 @@ def get_subdict(adict, keys):
     return dict((key, adict[key]) for key in keys if key in adict)
 
 def set_defaults(dict_, defaults):
-    for key, val in defaults.iteritems():
+    for key, val in six.iteritems(defaults):
         dict_.setdefault(key, val)
 
 ##
@@ -1334,7 +1402,7 @@ def check_names(names1, names2, msg):
 # c: 27.02.2008, r: 27.02.2008
 def select_by_names(objs_all, names, replace=None, simple=True):
     objs = {}
-    for key, val in objs_all.iteritems():
+    for key, val in six.iteritems(objs_all):
         if val.name in names:
             if replace is None:
                 objs[key] = val
@@ -1351,7 +1419,7 @@ def select_by_names(objs_all, names, replace=None, simple=True):
     return objs
 
 def ordered_iteritems(adict):
-    keys = adict.keys()
+    keys = list(adict.keys())
     order = nm.argsort(keys)
     for ii in order:
         key = keys[ii]
@@ -1362,7 +1430,7 @@ def dict_to_array(adict):
     Convert a dictionary of nD arrays of the same shapes with
     non-negative integer keys to a single (n+1)D array.
     """
-    keys = adict.keys()
+    keys = list(adict.keys())
     ik = nm.array(keys, dtype=nm.int32)
     assert_((ik >= 0).all())
 
@@ -1372,7 +1440,7 @@ def dict_to_array(adict):
     aux = nm.asarray(adict[ik[0]])
     out = nm.empty((ik.max() + 1,) + aux.shape, dtype=aux.dtype)
     out.fill(-1)
-    for key, val in adict.iteritems():
+    for key, val in six.iteritems(adict):
         out[key] = val
 
     return out
