@@ -18,11 +18,16 @@ from sfepy.base.parse_conf import create_bnf
 import six
 
 _required = ['filename_mesh|filename_domain', 'field_[0-9]+|fields',
-             'ebc_[0-9]+|ebcs', 'equations',
+            # TODO originaly EBC were required to be specified but in some examples
+            #  (especially 1D) only EPBCs specified is valid
+             'ebc_[0-9]+|ebcs|dgebc_[0-9]+|dgebcs|dgepbc_[0-9]+|dgepbcs',
+             'equations',
              'region_[0-9]+|regions', 'variable_[0-9]+|variables',
              'material_[0-9]+|materials',
              'solver_[0-9]+|solvers']
-_other = ['epbc_[0-9]+|epbcs', 'lcbc_[0-9]+|lcbcs', 'nbc_[0-9]+|nbcs',
+
+_other = ['epbc_[0-9]+|epbcs',
+          'lcbc_[0-9]+|lcbcs', 'nbc_[0-9]+|nbcs',
           'ic_[0-9]+|ics', 'function_[0-9]+|functions', 'options',
           'integral_[0-9]+|integrals']
 
@@ -84,6 +89,9 @@ def transform_conditions(adict, prefix):
 
     return d2
 
+def transform_dgebcs(adict):
+    return transform_conditions(adict, "dgebc")
+
 def transform_ebcs(adict):
     return transform_conditions(adict, 'ebc')
 
@@ -120,7 +128,7 @@ def transform_lcbcs(adict):
 
     return d2
 
-def transform_epbcs(adict):
+def transform_epbcs(adict, prefix="epbc"):
     d2 = {}
     for ii, (key, conf) in enumerate(six.iteritems(adict)):
         if isinstance(conf, tuple):
@@ -131,12 +139,15 @@ def transform_epbcs(adict):
                 c2 = tuple_to_conf(key, conf,
                                    ['region', 'times', 'dofs', 'match'])
 
-            d2['epbc_%s__%d' % (c2.name, ii)] = c2
+            d2['%s_%s__%d' % (prefix, c2.name, ii)] = c2
         else:
             c2 = transform_to_struct_1(conf)
-            d2['epbc_%s' % c2.name] = c2
+            d2['%s_%s' % (prefix, c2.name)] = c2
     return d2
 
+def transform_dgepbcs(adict):
+    return transform_epbcs(adict, "dgepbc")
+
 def transform_regions(adict):
     d2 = {}
     for ii, (key, conf) in enumerate(six.iteritems(adict)):
@@ -147,6 +158,9 @@ def transform_regions(adict):
             c2 = tuple_to_conf(key, conf, ['select', 'kind'])
             if len(conf) == 3:
                 c2.parent = conf[2]
+            if len(conf) == 4:
+                c2.parent = conf[2]
+                c2.extra_options = conf[3]
             d2['region_%s__%d' % (c2.name, ii)] = c2
         else:
             c2 = transform_to_struct_1(conf)
@@ -258,6 +272,8 @@ transforms = {
     'variables' : transform_variables,
     'ebcs'      : transform_ebcs,
     'epbcs'     : transform_epbcs,
+    'dgebcs'    : transform_dgebcs,
+    'dgepbcs'   : transform_dgepbcs,
     'nbcs'      : transform_to_struct_01,
     'lcbcs'     : transform_lcbcs,
     'ics'       : transform_ics,
@@ -265,7 +281,7 @@ transforms = {
     'functions' : transform_functions,
 }
 
-def dict_from_string(string):
+def dict_from_string(string, allow_tuple=False, free_word=False):
     """
     Parse `string` and return a dictionary that can be used to
     construct/override a ProblemConf instance.
@@ -276,7 +292,7 @@ def dict_from_string(string):
     if isinstance(string, dict):
         return string
 
-    parser = create_bnf()
+    parser = create_bnf(allow_tuple=allow_tuple, free_word=free_word)
 
     out = {}
     for r in parser.parseString(string, parseAll=True):
