@@ -16,13 +16,14 @@ import numpy as nm
 
 from sfepy.base.base import output, get_default, assert_
 from sfepy.base.base import Struct
+from sfepy.base.timing import Timer
 from sfepy.discrete.common.fields import parse_shape, Field
+from sfepy.discrete import PolySpace
 from sfepy.discrete.fem.mesh import Mesh
 from sfepy.discrete.fem.meshio import convert_complex_output
 from sfepy.discrete.fem.utils import (extend_cell_data, prepare_remap,
                                       invert_remap, get_min_value)
 from sfepy.discrete.fem.mappings import VolumeMapping, SurfaceMapping
-from sfepy.discrete.fem.poly_spaces import PolySpace
 from sfepy.discrete.fem.fe_surface import FESurface
 from sfepy.discrete.integrals import Integral
 from sfepy.discrete.fem.linearizer import (get_eval_dofs, get_eval_coors,
@@ -630,19 +631,19 @@ class FEField(Field):
         bf_key = (integral.order, _key, derivative)
 
         if bf_key not in self.bf:
-            if (iels is not None) and (self.ori is not None):
-                ori = self.ori[iels]
-
-            else:
-                ori = self.ori
-
+            ori = self.ori
             self.bf[bf_key] = ps.eval_base(qp.vals, diff=derivative, ori=ori,
                                            transform=self.basis_transform)
 
+        bf = self.bf[bf_key]
+        if iels is not None and bf.ndim == 4:
+            bf = bf[iels]
+
         if base_only:
-            return self.bf[bf_key]
+            return bf
+
         else:
-            return self.bf[bf_key], qp.weights
+            return bf, qp.weights
 
     def create_bqp(self, region_name, integral):
         gel = self.gel
@@ -885,8 +886,6 @@ class FEField(Field):
         cache : Struct instance
             The evaluate cache.
         """
-        import time
-
         try:
             from scipy.spatial import cKDTree as KDTree
         except ImportError:
@@ -897,7 +896,7 @@ class FEField(Field):
         if cache is None:
             cache = Struct(name='evaluate_cache')
 
-        tt = time.clock()
+        timer = Timer(start=True)
         if (cache.get('cmesh', None) is None) or not share_geometry:
             mesh = self.create_mesh(extra_nodes=False)
             cache.cmesh = cmesh = mesh.cmesh
@@ -917,13 +916,13 @@ class FEField(Field):
                 cache.normals0 = cmesh.get_facet_normals(0)
                 cache.normals1 = cmesh.get_facet_normals(1)
 
-        output('cmesh setup: %f s' % (time.clock()-tt), verbose=verbose)
+        output('cmesh setup: %f s' % timer.stop(), verbose=verbose)
 
-        tt = time.clock()
+        timer.start()
         if (cache.get('kdtree', None) is None) or not share_geometry:
             cache.kdtree = KDTree(cmesh.coors)
 
-        output('kdtree: %f s' % (time.clock()-tt), verbose=verbose)
+        output('kdtree: %f s' % timer.stop(), verbose=verbose)
 
         return cache
 
@@ -996,11 +995,14 @@ class FEField(Field):
         coors = domain.get_mesh_coors(actual=True)
         dconn = domain.get_conn()
 
+        tco = integration in ('volume', 'custom')
+        iels = region.get_cells(true_cells_only=tco)
+        transform = (self.basis_transform[iels] if self.basis_transform
+                     is not None else None)
+
         if integration == 'volume':
             qp = self.get_qp('v', integral)
 
-            iels = region.get_cells()
-
             geo_ps = self.gel.poly_space
             ps = self.poly_space
             bf = self.get_base('v', 0, integral, iels=iels)
@@ -1008,8 +1010,7 @@ class FEField(Field):
             conn = nm.take(dconn, iels.astype(nm.int32), axis=0)
             mapping = VolumeMapping(coors, conn, poly_space=geo_ps)
             vg = mapping.get_mapping(qp.vals, qp.weights, poly_space=ps,
-                                     ori=self.ori,
-                                     transform=self.basis_transform)
+                                     ori=self.ori, transform=transform)
 
             out = vg
 
@@ -1021,6 +1022,11 @@ class FEField(Field):
                       ' hierarchical basis!'
                 raise ValueError(msg)
 
+            if self.basis_transform is not None:
+                msg = 'surface integrals do not work with the' \
+                      ' basis transform!'
+                raise ValueError(msg)
+
             sd = domain.surface_groups[region.name]
             esd = self.surface_data[region.name]
 
@@ -1034,7 +1040,7 @@ class FEField(Field):
                 self.create_bqp(region.name, integral)
                 qp = self.qp_coors[(integral.order, esd.bkey)]
 
-                abf = ps.eval_base(qp.vals[0], transform=self.basis_transform)
+                abf = ps.eval_base(qp.vals[0], transform=transform)
                 bf = abf[..., self.efaces[0]]
 
                 indx = self.gel.get_surface_entities()[0]
@@ -1055,9 +1061,14 @@ class FEField(Field):
                     sg.evaluate_bfbgm(bf_bg, ebf_bg, coors, sd.fis, dconn)
 
             else:
+                if self.basis_transform is not None:
+                    msg = 'surface fields do not work with the' \
+                          ' basis transform!'
+                    raise ValueError(msg)
+
                 # Do not use BQP for surface fields.
                 qp = self.get_qp(sd.face_type, integral)
-                bf = ps.eval_base(qp.vals, transform=self.basis_transform)
+                bf = ps.eval_base(qp.vals, transform=transform)
 
                 sg = mapping.get_mapping(qp.vals, qp.weights,
                                          poly_space=Struct(n_nod=bf.shape[-1]),
@@ -1191,8 +1202,9 @@ class VolumeField(FEField):
 
         if (dct == 'surface') or (geometry_flag):
             reg = info.get_region()
+            mreg_name = info.get_region_name(can_trace=False)
             self.domain.create_surface_group(reg)
-            self.setup_surface_data(reg, is_trace)
+            self.setup_surface_data(reg, is_trace, mreg_name)
 
         elif dct == 'edge':
             raise NotImplementedError('dof connectivity type %s' % dct)
@@ -1209,7 +1221,7 @@ class VolumeField(FEField):
             conn.shape += (1,)
             self.point_data[region.name] = conn
 
-    def setup_surface_data(self, region, is_trace=False):
+    def setup_surface_data(self, region, is_trace=False, trace_region=None):
         """nodes[leconn] == econn"""
         """nodes are sorted by node number -> same order as region.vertices"""
         if region.name not in self.surface_data:
@@ -1219,7 +1231,7 @@ class VolumeField(FEField):
 
         if region.name in self.surface_data and is_trace:
             sd = self.surface_data[region.name]
-            sd.setup_mirror_connectivity(region)
+            sd.setup_mirror_connectivity(region, trace_region)
 
         return self.surface_data[region.name]
 
@@ -1365,7 +1377,8 @@ class SurfaceField(FEField):
 
         if reg.name in self.surface_data and is_trace:
             sd = self.surface_data[reg.name]
-            sd.setup_mirror_connectivity(reg)
+            mreg_name = info.get_region_name(can_trace=False)
+            sd.setup_mirror_connectivity(reg, mreg_name)
 
     def _init_econn(self):
         """
