@@ -2,14 +2,18 @@
 Operators for enforcing linear combination boundary conditions in nodal FEM
 setting.
 """
+from __future__ import absolute_import
 import numpy as nm
 import scipy.sparse as sp
 
-from sfepy.base.base import output, assert_, find_subclasses, Container, Struct
+from sfepy.base.base import (basestr, output, assert_, find_subclasses,
+                             Container, Struct)
 from sfepy.discrete.common.dof_info import DofInfo, expand_nodes_to_equations
 from sfepy.discrete.fem.utils import (compute_nodal_normals,
                                       compute_nodal_edge_dirs)
-from sfepy.discrete.conditions import get_condition_value
+from sfepy.discrete.conditions import get_condition_value, Function
+import six
+from six.moves import range
 
 class LCBCOperator(Struct):
     """
@@ -35,6 +39,12 @@ class LCBCOperator(Struct):
 class MRLCBCOperator(LCBCOperator):
     """
     Base class for model-reduction type LCBC operators.
+
+    These operators are applied to a single field, and replace its DOFs in a
+    given region by new DOFs. In case some field DOFs are to be preserved,
+    those have to be "copied" explicitly, by setting the corresponding row of
+    the operator matrix to a single value one (see, for example,
+    :class:`NoPenetrationOperator`).
     """
 
     def __init__(self, name, regions, dof_names, dof_map_fun, variables,
@@ -58,16 +68,13 @@ class MRLCBCOperator(LCBCOperator):
     def setup(self):
         eq = self.eq_map.eq
         meq = expand_nodes_to_equations(self.mdofs, self.dof_names,
-                                         self.all_dof_names)
-        ameq = eq[meq]
-        assert_(nm.all(ameq >= 0))
+                                        self.all_dof_names)
+        self.ameq = eq[meq]
+        assert_(nm.all(self.ameq >= 0))
 
         if self.eq_map.n_epbc:
             self.treat_pbcs(meq, self.eq_map.master)
 
-        # Node-by-node order compatible with MRLCBCOperator matrices.
-        self.ameq = ameq.reshape((len(self.dof_names), -1)).T.ravel()
-
     def treat_pbcs(self, dofs, master):
         """
         Treat dofs with periodic BC.
@@ -177,7 +184,7 @@ class NoPenetrationOperator(MRLCBCOperator):
         data = []
         rows = []
         cols = []
-        for idim in xrange(dim):
+        for idim in range(dim):
             ic = nm.where(ii == idim)[0]
             if len(ic) == 0: continue
 
@@ -187,8 +194,8 @@ class NoPenetrationOperator(MRLCBCOperator):
                 nn[:,ik] = - normals[ic,il] / normals[ic,idim]
 
             irn = dim * ic + idim
-            ics = [(dim - 1) * ic + ik for ik in xrange(dim - 1)]
-            for ik in xrange(dim - 1):
+            ics = [(dim - 1) * ic + ik for ik in range(dim - 1)]
+            for ik in range(dim - 1):
                 rows.append(irn)
                 cols.append(ics[ik])
                 data.append(nn[:,ik])
@@ -259,7 +266,7 @@ class NormalDirectionOperator(MRLCBCOperator):
         return normals
 
 class EdgeDirectionOperator(NormalDirectionOperator):
-    """
+    r"""
     Transformation matrix operator for edges direction LCBCs.
 
     The substitution (in 3D) is:
@@ -283,7 +290,7 @@ class EdgeDirectionOperator(NormalDirectionOperator):
 class IntegralMeanValueOperator(MRLCBCOperator):
     """
     Transformation matrix operator for integral mean value LCBCs.
-    All node DOFs are sumed to the new one.
+    All DOFs in a region are summed to form a single new DOF.
     """
     kind = 'integral_mean_value'
 
@@ -299,15 +306,148 @@ class IntegralMeanValueOperator(MRLCBCOperator):
         rows = nm.arange(data.shape[0])
         cols = nm.zeros((data.shape[0],))
 
-        mtx = sp.coo_matrix((data, (rows, cols)), shape=(n_nod * dpn, dpn))
+        mtx = sp.coo_matrix((data, (rows, cols)), shape=(n_nod * dpn, 1))
 
         self.n_mdof = n_nod * dpn
-        self.n_new_dof = dpn
+        self.n_new_dof = 1
         self.mtx = mtx.tocsr()
 
+class NodalLCOperator(MRLCBCOperator):
+    r"""
+    Transformation matrix operator for the general linear combination of DOFs
+    in each node of a field in the given region.
+
+    The DOFs can be fully constrained - then the operator corresponds to
+    enforcing Dirichlet boundary conditions.
+
+    The linear combination is given by:
+
+    .. math::
+        \sum_{j=1}^n A_{ij} u_j = b_i \;,\ \forall i \;,
+
+    where :math:`u_j`, :math:`j = 1, \dots, n` are the DOFs in the node and
+    :math:`i = 1, \dots, m`, :math:`m < n`, are the linear constraint indices.
+
+    SymPy is used to solve the constraint linear system in each node for the
+    dependent DOF(s).
+    """
+    kind = 'nodal_combination'
+
+    def __init__(self, name, regions, dof_names, dof_map_fun,
+                 constraints, variables, ts=None, functions=None):
+        MRLCBCOperator.__init__(self, name, regions, dof_names, dof_map_fun,
+                                variables, functions=functions)
+        import sympy as sm
+
+        n_c = self.field.n_components
+        dpn = len(self.dof_names)
+        n_nod = self.mdofs.shape[0]
+
+        assert_(dpn <= n_c)
+
+        if (isinstance(constraints, basestr)
+            or isinstance(constraints, Function)):
+            fun = get_condition_value(constraints, functions,
+                                      'nodal', 'constraints')
+            coors = self.field.get_coor(self.mdofs)
+            mtx, rhs = fun(ts, coors, self.region)
+
+        else:
+            mtx, rhs = constraints
+            mtx = nm.tile(mtx, (n_nod, 1, 1))
+            rhs = nm.tile(rhs, (n_nod, 1))
+
+        n_ceq = mtx.shape[1]
+
+        assert_(n_ceq == rhs.shape[1])
+        assert_(dpn == mtx.shape[2])
+        assert_(n_ceq <= dpn)
+
+        data = []
+        rows = []
+        cols = []
+        rhss = nm.zeros(n_nod * dpn, dtype=nm.float64)
+        n_new = 0
+        us = [sm.Symbol('u%d' % ii) for ii in range(dpn)]
+        for im, nmtx in enumerate(mtx):
+            eqs = sm.Matrix(nmtx) * sm.Matrix(us) - rhs[im][:, None]
+            sol = sm.solve(eqs)
+
+            assert_(len(sol) == n_ceq)
+
+            imasters = []
+            ifixed = []
+            islaves = set()
+            ccs = []
+            for key, _poly in six.iteritems(sol):
+                imaster = int(key.name[1:])
+                imasters.append(imaster)
+
+                if not isinstance(_poly, sm.Float):
+                    poly = _poly.as_poly()
+
+                    # Workaround for poly.all_coeffs() not applicable to
+                    # multivariate polynomials.
+                    coefs = []
+                    for ii, uu in enumerate(us):
+                        if poly.has(uu):
+                            coefs.append(poly.coeff_monomial(uu))
+                            islaves.add(ii)
+                    coefs.append(poly.TC())
+                    ccs.append(coefs)
+
+                else: # Degenerated constraint - fixed master.
+                    ifixed.append(imaster)
+                    ccs.append([float(_poly)])
+
+            islaves = sorted(islaves)
+
+            for ii, imaster in enumerate(imasters):
+                coefs = ccs[ii]
+
+                em = dpn * im + imaster
+                rhss[em] = coefs[-1]
+
+                if imaster in ifixed: continue
+
+                # Master DOF is expressed in terms of slave DOFs.
+                for ii, islave in enumerate(islaves):
+                    es = ii + n_new
+
+                    rows.append(em)
+                    cols.append(es)
+                    data.append(coefs[ii])
+
+            # Slave DOFs are copied.
+            for ii, islave in enumerate(islaves):
+                em = dpn * im + islave
+                es = ii + n_new
+
+                rows.append(em)
+                cols.append(es)
+                data.append(1.0)
+
+            n_new += len(islaves)
+
+        rows = nm.array(rows, dtype=nm.int32)
+        cols = nm.array(cols, dtype=nm.int32)
+        data = nm.array(data, dtype=nm.float64)
+
+        mtx = sp.coo_matrix((data, (rows, cols)), shape=(n_nod * dpn, n_new))
+
+        self.n_mdof = n_nod * dpn
+        self.n_new_dof = n_new
+        self.mtx = mtx.tocsr()
+
+        self.rhs = rhss
+
 class ShiftedPeriodicOperator(LCBCOperator):
     """
-    Transformation matrix operator shifted periodic boundary conditions.
+    Transformation matrix operator for shifted periodic boundary conditions.
+
+    This operator ties existing DOFs of two fields in two disjoint regions
+    together. Unlike :class:`MRLCBCOperator` subclasses, it does not create any
+    new DOFs.
     """
     kind = 'shifted_periodic'
 
@@ -359,7 +499,7 @@ class ShiftedPeriodicOperator(LCBCOperator):
         num = len(ia)
 
         ones = nm.ones(num, dtype=nm.float64)
-        n_dofs = [variables.adi.n_dof[name] for name in self.var_names]
+        n_dofs = [variables.adi.n_dof[ii] for ii in self.var_names]
         mtx = sp.coo_matrix((ones, (meq, seq)), shape=n_dofs)
 
         self.mtx = mtx.tocsr()
@@ -459,20 +599,21 @@ class LCBCOperators(Container):
             ics.setdefault(op.var_names[0], []).append((ii, op.n_new_dof))
 
         self.ics = {}
-        for key, val in ics.iteritems():
+        for key, val in six.iteritems(ics):
             iis, ics = zip(*val)
             self.ics[key] = (iis, nm.cumsum(nm.r_[0, ics]))
 
         self.n_free = {}
         self.n_active = {}
         n_dof = self.variables.adi.n_dof
-        for key in self.n_master.iterkeys():
+        for key in six.iterkeys(self.n_master):
             self.n_free[key] = n_dof[key] - self.n_master[key]
             self.n_active[key] = self.n_free[key] + self.n_new[key]
 
         def _dict_to_di(name, dd):
             di = DofInfo(name)
-            for key, val in dd.iteritems():
+            for key in keys:
+                val = dd[key]
                 di.append_raw(key, val)
             return di
 
@@ -505,10 +646,10 @@ class LCBCOperators(Container):
         if len(self) == 0: return (None,) * 3
 
         n_dof = self.variables.adi.ptr[-1]
-        n_constrained = nm.sum([val for val in self.n_master.itervalues()])
-        n_dof_free = nm.sum([val for val in self.n_free.itervalues()])
-        n_dof_new = nm.sum([val for val in self.n_new.itervalues()])
-        n_dof_active = nm.sum([val for val in self.n_active.itervalues()])
+        n_constrained = nm.sum([val for val in six.itervalues(self.n_master)])
+        n_dof_free = nm.sum([val for val in six.itervalues(self.n_free)])
+        n_dof_new = nm.sum([val for val in six.itervalues(self.n_new)])
+        n_dof_active = nm.sum([val for val in six.itervalues(self.n_active)])
 
         output('dofs: total %d, free %d, constrained %d, new %d'\
                % (n_dof, n_dof_free, n_constrained, n_dof_new))
@@ -579,7 +720,8 @@ class LCBCOperators(Container):
                 cols.append(coff + ii2)
                 data.append(lv)
 
-                vec_lc[irs] += op.rhs
+            if vec_lc is not None:
+                vec_lc[irs] += op.get('rhs', 0)
 
         rows = nm.concatenate(rows)
         cols = nm.concatenate(cols)
