@@ -2,7 +2,8 @@ from __future__ import absolute_import
 import numpy as nm
 import numpy.linalg as nla
 
-from sfepy.base.base import find_subclasses, assert_, Struct
+from sfepy.base.base import assert_, Struct
+from sfepy.discrete import PolySpace
 from sfepy.linalg import combine, insert_strided_axis
 from six.moves import range
 from functools import reduce
@@ -21,27 +22,8 @@ vertex_maps = {3 : [[0, 0, 0],
                     [1, 1],
                     [0, 1]],
                1 : [[0],
-                    [1]]}
-
-def transform_basis(transform, bf):
-    """
-    Transform a basis `bf` using `transform` array of matrices.
-    """
-    if bf.ndim == 3:
-        nbf = nm.einsum('cij,qdj->cqdi', transform, bf, order='C')
-
-    elif bf.ndim == 4:
-        if bf.shape[0] == 1:
-            nbf = nm.einsum('cij,qdj->cqdi', transform, bf[0], order='C')
-
-        else:
-            nbf = nm.einsum('cij,cqdj->cqdi', transform, bf, order='C')
-
-    # Note: the 2nd derivatives are not supported here.
-    # Workaround for NumPy 1.14.0 - order is ignored(?)
-    nbf = nm.ascontiguousarray(nbf)
-
-    return nbf
+                    [1]],
+               0 : [[0]]}
 
 class LagrangeNodes(Struct):
     """Helper class for defining nodes of Lagrange elements."""
@@ -122,7 +104,7 @@ class LagrangeNodes(Struct):
     @staticmethod
     def append_tp_faces(nodes, nts, iseq, nt, faces, ao):
         delta = 1.0 / (float(ao) ** 2)
-        for ii, face in enumerate( faces ):
+        for ii, face in enumerate(faces):
             n1 = nodes[face[0],:].copy()
             n2 = nodes[face[1],:].copy()
             n3 = nodes[face[2],:].copy()
@@ -243,148 +225,17 @@ class NodeDescription(Struct):
         """
         return (self.n_edge_nod + self.n_face_nod + self.n_bubble_nod) > 0
 
-class PolySpace(Struct):
-    """Abstract polynomial space class."""
-    _all = None
-
-    keys = {
-        (1, 2) : 'simplex',
-        (2, 3) : 'simplex',
-        (3, 4) : 'simplex',
-        (2, 4) : 'tensor_product',
-        (3, 8) : 'tensor_product',
-    }
-
-    @staticmethod
-    def any_from_args(name, geometry, order, base='lagrange',
-                      force_bubble=False):
-        """
-        Construct a particular polynomial space classes according to the
-        arguments passed in.
-        """
-        if name is None:
-            name = PolySpace.suggest_name(geometry, order, base, force_bubble)
-
-        if PolySpace._all is None:
-            PolySpace._all = find_subclasses(globals(), [PolySpace])
-        table = PolySpace._all
-
-        key = '%s_%s' % (base, PolySpace.keys[(geometry.dim,
-                                               geometry.n_vertex)])
-        if (geometry.name == '1_2') and (key not in table):
-            key = '%s_%s' % (base, 'tensor_product')
-
-        if force_bubble:
-            key += '_bubble'
-
-        return table[key](name, geometry, order)
-
-    @staticmethod
-    def suggest_name(geometry, order, base='lagrange',
-                     force_bubble=False):
-        """
-        Suggest the polynomial space name given its constructor parameters.
-        """
-        aux = geometry.get_interpolation_name()[:-1]
-        if force_bubble:
-            return aux + ('%dB' % order)
-        else:
-            return aux + ('%d' % order)
-
-    def __init__(self, name, geometry, order):
-        self.name = name
-        self.geometry = geometry
-        self.order = order
-
-        self.bbox = nm.vstack((geometry.coors.min(0), geometry.coors.max(0)))
-
-    def eval_base(self, coors, diff=0, ori=None, force_axis=False,
-                  transform=None, suppress_errors=False, eps=1e-15):
-        """
-        Evaluate the basis or its first or second derivatives in points given
-        by coordinates. The real work is done in _eval_base() implemented in
-        subclasses.
-
-        Note that the second derivative code is a work-in-progress and only
-        `coors` and `transform` arguments are used.
-
-        Parameters
-        ----------
-        coors : array_like
-            The coordinates of points where the basis is evaluated. See Notes.
-        diff : 0, 1 or 2
-            If nonzero, return the given derivative.
-        ori : array_like, optional
-            Optional orientation of element facets for per element basis.
-        force_axis : bool
-            If True, force the resulting array shape to have one more axis even
-            when `ori` is None.
-        transform : array_like, optional
-            The basis transform array.
-        suppress_errors : bool
-            If True, do not report points outside the reference domain.
-        eps : float
-            Accuracy for comparing coordinates.
-
-        Returns
-        -------
-        base : array
-            The basis (shape (n_coor, 1, n_base)) or its first derivative
-            (shape (n_coor, dim, n_base)) or its second derivative (shape
-            (n_coor, dim, dim, n_base)) evaluated in the given points. An
-            additional axis is pre-pended of length n_cell, if `ori` is given,
-            or of length 1, if `force_axis` is True.
-
-        Notes
-        -----
-        If coors.ndim == 3, several point sets are assumed, with equal number
-        of points in each of them. This is the case, for example, of the
-        values of the volume base functions on the element facets. The indexing
-        (of bf_b(g)) is then (ifa,iqp,:,n_ep), so that the facet can be set in
-        C using FMF_SetCell.
-        """
-        coors = nm.asarray(coors)
-        if not coors.ndim in (2, 3):
-            raise ValueError('coordinates must have 2 or 3 dimensions! (%d)'
-                             % coors.ndim)
-
-        if (coors.ndim == 2):
-            base = self._eval_base(coors, diff=diff, ori=ori,
-                                   suppress_errors=suppress_errors,
-                                   eps=eps)
-
-            if (base.ndim == 3) and force_axis:
-                base = base[None, ...]
-
-            if not base.flags['C_CONTIGUOUS']:
-                base = nm.ascontiguousarray(base)
-
-        else: # Several point sets.
-            if diff:
-                bdim = self.geometry.dim
-            else:
-                bdim = 1
-
-            base = nm.empty((coors.shape[0], coors.shape[1],
-                             bdim, self.n_nod), dtype=nm.float64)
-
-            for ii, _coors in enumerate(coors):
-                base[ii] = self._eval_base(_coors, diff=diff, ori=ori,
-                                           suppress_errors=suppress_errors,
-                                           eps=eps)
-
-        if transform is not None:
-            base = transform_basis(transform, base)
-
-        return base
-
+class FEPolySpace(PolySpace):
+    """
+    Base for FE polynomial space classes.
+    """
     def get_mtx_i(self):
         return self.mtx_i
 
     def describe_nodes(self):
         return NodeDescription(self.nts, self.nodes)
 
-class LagrangePolySpace(PolySpace):
+class LagrangePolySpace(FEPolySpace):
 
     def create_context(self, cmesh, eps, check_errors, i_max, newton_eps,
                        tdim=None):
@@ -487,7 +338,10 @@ class LagrangeSimplexPolySpace(LagrangePolySpace):
             nodes[iseq:iseq+n_v,:] = aux
             iseq += n_v
 
-            if dim == 1:
+            if dim == 0:
+                pass
+
+            elif dim == 1:
                 iseq = LagrangeNodes.append_edges(nodes, nts, iseq, 3,
                                                   [[0, 1]], order)
             elif dim == 2:
@@ -681,8 +535,7 @@ class LagrangeTensorProductPolySpace(LagrangePolySpace):
 
             # Vertex nodes.
             nts[0:n_v,0] = 0
-            nts[0:n_v,1] = nm.arange( n_v, dtype = nm.int32 )
-            order * nm.identity( n_v, dtype = nm.int32 )
+            nts[0:n_v,1] = nm.arange(n_v, dtype=nm.int32)
             if dim == 3:
                 for ii in range(n_v):
                     i1, i2, i3 = vertex_map[ii]
@@ -732,7 +585,7 @@ class LagrangeTensorProductPolySpace(LagrangePolySpace):
 
         else:
             c_min, c_max = self.bbox[:,0]
-            
+
             cr = nm.arange(2 * dim)
             node_coors = (nodes[:,cr[::2]] * c_min
                           + nodes[:,cr[1::2]] * c_max) / order
@@ -745,7 +598,7 @@ class LagrangeTensorProductPolySpace(LagrangePolySpace):
         dim = self.geometry.dim
 
         ev = self.ps1d.eval_base
-        
+
         if diff:
             base = nm.ones((coors.shape[0], dim, self.n_nod), dtype=nm.float64)
 
@@ -759,7 +612,7 @@ class LagrangeTensorProductPolySpace(LagrangePolySpace):
                                                 diff=True,
                                                 suppress_errors=suppress_errors,
                                                 eps=eps)
-                
+
                     else:
                         base[:,iv:iv+1,:] *= ev(coors[:,ii:ii+1].copy(),
                                                 diff=False,
@@ -772,7 +625,7 @@ class LagrangeTensorProductPolySpace(LagrangePolySpace):
             for ii in range(dim):
                 self.ps1d.nodes = self.nodes[:,2*ii:2*ii+2].copy()
                 self.ps1d.n_nod = self.n_nod
-                
+
                 base *= ev(coors[:,ii:ii+1].copy(),
                            diff=diff,
                            suppress_errors=suppress_errors,
@@ -825,7 +678,143 @@ class LagrangeTensorProductPolySpace(LagrangePolySpace):
     def get_mtx_i(self):
         return self.ps1d.mtx_i
 
-class LobattoTensorProductPolySpace(PolySpace):
+class SerendipityTensorProductPolySpace(FEPolySpace):
+    """
+    Serendipity polynomial space using Lagrange functions.
+
+    Notes
+    -----
+    - Orders >= 4 (with bubble functions) are not supported.
+    - Does not use CLagrangeContext, basis functions are hardcoded.
+    - `self.nodes`, `self.node_coors` are not used for basis evaluation and
+       assembling.
+    """
+    name = 'serendipity_tensor_product'
+    supported_orders = {1, 2, 3}
+
+    from sfepy.discrete.fem._serendipity import all_bfs
+
+    def __init__(self, name, geometry, order):
+        import sympy as sm
+
+        if geometry.dim < 2:
+            raise ValueError('serendipity elements need dimension 2 or 3! (%d)'
+                             % geometry.dim)
+        if order not in self.supported_orders:
+            raise ValueError('serendipity elements support only orders %s! (%d)'
+                             % (self.supported_orders, order))
+        PolySpace.__init__(self, name, geometry, order)
+
+        self.nodes, self.nts, self.node_coors = self._define_nodes()
+        self.n_nod = self.nodes.shape[0]
+
+        bfs = self.all_bfs[geometry.dim][order]
+        self.bfs = bfs[0]
+        self.bfgs = bfs[1]
+
+        x, y, z = sm.symbols('x y z')
+        vs = [x, y, z][:geometry.dim]
+
+        self._bfs = [sm.lambdify(vs, bf) for bf in self.bfs]
+        self._bfgs = [[sm.lambdify(vs, bfg) for bfg in bfgs]
+                      for bfgs in self.bfgs]
+
+    def create_context(self, cmesh, eps, check_errors, i_max, newton_eps,
+                       tdim=None):
+        pass
+
+    def _define_nodes(self):
+        geometry = self.geometry
+        order = self.order
+
+        n_v, dim = geometry.n_vertex, geometry.dim
+
+        vertex_map = order * nm.array(vertex_maps[dim], dtype=nm.int32)
+
+        # Only for orders 1, 2, 3!
+        if dim == 2:
+            n_nod = 4 * self.order
+
+        else:
+            n_nod = 8 + 12 * (self.order - 1)
+
+        nodes = nm.zeros((n_nod, 2 * dim), nm.int32)
+        nts = nm.zeros((n_nod, 2), nm.int32)
+
+        if order == 0:
+            nts[0, :] = [3, 0]
+            nodes[0, :] = nm.zeros((n_nod,), nm.int32)
+
+        else:
+            iseq = 0
+
+            # Vertex nodes.
+            nts[0:n_v, 0] = 0
+            nts[0:n_v, 1] = nm.arange(n_v, dtype=nm.int32)
+
+            if dim == 3:
+                for ii in range(n_v):
+                    i1, i2, i3 = vertex_map[ii]
+                    nodes[iseq, :] = [order - i1, i1,
+                                      order - i2, i2,
+                                      order - i3, i3]
+                    iseq += 1
+
+            else: # dim == 2:
+                for ii in range(n_v):
+                    i1, i2 = vertex_map[ii]
+                    nodes[iseq, :] = [order - i1, i1, order - i2, i2]
+                    iseq += 1
+
+            if dim == 2:
+                iseq = LagrangeNodes.append_tp_edges(nodes, nts, iseq, 1,
+                                                     geometry.edges, order)
+
+            elif dim == 3:
+                iseq = LagrangeNodes.append_tp_edges(nodes, nts, iseq, 1,
+                                                     geometry.edges, order)
+
+            else:
+                raise NotImplementedError
+
+        # Coordinates of the nodes.
+        c_min, c_max = self.bbox[:, 0]
+
+        cr = nm.arange(2 * dim)
+        node_coors = (nodes[:, cr[::2]] * c_min
+                      + nodes[:, cr[1::2]] * c_max) / order
+
+        return nodes, nts, nm.ascontiguousarray(node_coors)
+
+    def _eval_base(self, coors, diff=0, ori=None,
+                   suppress_errors=False, eps=1e-15):
+        """
+        See :func:`PolySpace.eval_base()`.
+        """
+        dim = self.geometry.dim
+        if diff:
+            bdim = dim
+
+        else:
+            bdim = 1
+
+        base = nm.empty((coors.shape[0], bdim, self.n_nod), dtype=nm.float64)
+
+        if diff == 0:
+            for ib, bf in enumerate(self._bfs):
+                base[:, 0, ib] = bf(*coors.T)
+
+        elif diff == 1:
+            for ib, bfg in enumerate(self._bfgs):
+                for ig in range(dim):
+                    base[:, ig, ib] = bfg[ig](*coors.T)
+
+        else:
+            raise NotImplementedError
+
+        return base
+
+class LobattoTensorProductPolySpace(FEPolySpace):
     """
     Hierarchical polynomial space using Lobatto functions.
 
