@@ -1,12 +1,14 @@
 """
 Fields for isogeometric analysis.
 """
+from __future__ import absolute_import
 import numpy as nm
 
 from sfepy.base.base import assert_, basestr, Struct
 from sfepy.discrete.common.fields import parse_shape, Field
 from sfepy.discrete.iga.mappings import IGMapping
 from sfepy.discrete.iga.iga import get_bezier_element_entities
+from six.moves import range
 
 def parse_approx_order(approx_order):
     if (approx_order is None): return 0
@@ -134,11 +136,10 @@ class IGField(Field):
         ----------
         integral : Integral instance
             The integral describing used numerical quadrature.
-        integration : 'volume', 'plate', 'surface', 'surface_extra' or 'point'
-            The term integration type.
+        integration : 'volume'
+            The term integration type. Only 'volume' type is implemented.
         region_name : str
-            The name of surface region, required when `shape_kind` is
-            'surface'.
+            The name of the region of the integral.
 
         Returns
         -------
@@ -230,7 +231,7 @@ class IGField(Field):
         vals = []
 
         aux = self.get_dofs_in_region(region)
-        nods = nm.unique(nm.hstack(aux))
+        nods = nm.unique(aux)
 
         if nm.isscalar(fun):
             vals = nm.repeat([fun], nods.shape[0] * dpn)
@@ -318,7 +319,7 @@ class IGField(Field):
                 dets = all_dets[ii]
 
                 # Local projection system.
-                for idof in xrange(dpn):
+                for idof in range(dpn):
                     lrhs = (fbfs * (fvals[idof, :, None] * dets)).sum(0)
                     rhs[idof, elc] += lrhs
 
@@ -335,16 +336,18 @@ class IGField(Field):
             mvals = nm.concatenate(mvals)
             mtx = sps.coo_matrix((mvals, (rows, cols)), shape=(n_dof, n_dof))
 
-            vals = nm.zeros((dpn, n_dof), dtype=nm.float64)
+            vals = nm.zeros((n_dof, dpn), dtype=nm.float64)
 
             # Solve l2 projection system.
-            for idof in xrange(dpn):
+            for idof in range(dpn):
                 dofs = solve(mtx, rhs[idof, :])
-                vals[idof, remap[nods]] = dofs
+                vals[remap[nods], idof] = dofs
 
         else:
             raise ValueError('unknown function/value type! (%s)' % type(fun))
 
+        vals.shape = (len(nods), -1)
+
         return nods, vals
 
     def setup_extra_data(self, geometry, info, is_trace):
