@@ -1,19 +1,20 @@
 from __future__ import absolute_import
 import os
 import os.path as op
-import time
 from copy import copy
 
 import numpy as nm
 
 from sfepy.base.base import (
-    dict_from_keys_init, select_by_names, is_integer, is_sequence,
+    dict_from_keys_init, select_by_names, is_string, is_integer, is_sequence,
     output, get_default, Struct, IndexedStruct)
 import sfepy.base.ioutils as io
 from sfepy.base.conf import ProblemConf, get_standard_keywords
 from sfepy.base.conf import transform_variables, transform_materials
+from sfepy.base.timing import Timer
 from .functions import Functions
 from sfepy.discrete.fem.mesh import Mesh
+from sfepy.discrete.fem.meshio import check_format_suffix
 from sfepy.discrete.fem.fields_base import set_mesh_coors
 from sfepy.discrete.common.fields import fields_from_conf
 from .variables import Variables, Variable
@@ -26,6 +27,7 @@ from sfepy.discrete.evaluate import create_evaluable, eval_equations
 from sfepy.solvers.ts import TimeStepper
 from sfepy.discrete.evaluate import Evaluator
 from sfepy.solvers import Solver, NonlinearSolver
+from sfepy.solvers.solvers import use_first_available
 from sfepy.solvers.ts_solvers import StationarySolver
 import six
 from six.moves import range
@@ -52,7 +54,7 @@ def make_is_save(options):
                                                   self.save_times0)
 
         def __call__(self, ts):
-            if self.save_times == 'all':
+            if is_string(self.save_times) and self.save_times == 'all':
                 return True
 
             elif isinstance(self.save_times, nm.ndarray):
@@ -205,6 +207,12 @@ class Problem(Struct):
             if conf.options.get('ulf', False):
                 domain.mesh.coors_act = domain.mesh.coors.copy()
 
+            if conf.options.get('mesh_eps') is not None:
+                import sfepy.discrete.fem.mesh as msh
+                import sfepy.discrete.fem.periodic as per
+                msh.set_accuracy(conf.options.mesh_eps)
+                per.set_accuracy(conf.options.mesh_eps)
+
         elif conf.get('filename_domain') is not None:
             from sfepy.discrete.iga.domain import IGDomain
             domain = IGDomain.from_file(conf.filename_domain)
@@ -244,6 +252,7 @@ class Problem(Struct):
 
         self.reset()
         self.ls_conf = self.nls_conf = self.ts_conf = None
+        self.conf_variables = self.conf_materials = None
 
         if auto_conf:
             if equations is None:
@@ -309,12 +318,6 @@ class Problem(Struct):
                     hook = self.conf.get_function(hook)
                     setattr(self, hook_name, hook)
 
-        iter_hook = self.nls_iter_hook
-        if iter_hook is not None:
-            self.nls_iter_hook = lambda *args, **kwargs: \
-                                 iter_hook(self, *args, **kwargs)
-
-
     def copy(self, name=None):
         """
         Make a copy of Problem.
@@ -336,6 +339,7 @@ class Problem(Struct):
         obj.setup_output(output_filename_trunk=self.ofn_trunk,
                          output_dir=self.output_dir,
                          output_format=self.output_format,
+                         file_format=self.file_format,
                          file_per_var=self.file_per_var,
                          linearization=self.linearization)
 
@@ -391,32 +395,39 @@ class Problem(Struct):
         else:
             default_output_format = conf.options.get('output_format', None)
 
+        default_file_format = conf.options.get('file_format', None)
         default_file_per_var = conf.options.get('file_per_var', None)
         default_float_format = conf.options.get('float_format', None)
         default_linearization = Struct(kind='strip')
 
         self.setup_output(output_filename_trunk=default_trunk,
                           output_dir=default_output_dir,
-                          file_per_var=default_file_per_var,
                           output_format=default_output_format,
+                          file_format=default_file_format,
                           float_format=default_float_format,
+                          file_per_var=default_file_per_var,
                           linearization=default_linearization)
 
     def setup_output(self, output_filename_trunk=None, output_dir=None,
-                     output_format=None, float_format=None,
+                     output_format=None, file_format=None, float_format=None,
                      file_per_var=None, linearization=None):
         """
         Sets output options to given values, or uses the defaults for
         each argument that is None.
         """
-        self.output_modes = {'vtk' : 'sequence', 'h5' : 'single'}
+        self.output_modes = {'vtk' : 'sequence', 'h5' : 'single',
+                             'msh' : 'sequence'}
 
         self.ofn_trunk = get_default(output_filename_trunk,
-                                     io.get_trunk(self.domain.name))
+                                     op.basename(self.domain.name))
 
         self.set_output_dir(output_dir)
 
         self.output_format = get_default(output_format, 'vtk')
+        self.file_format = file_format
+        if self.file_format is not None:
+            check_format_suffix(self.file_format, self.output_format)
+
         self.float_format = get_default(float_format, None)
         self.file_per_var = get_default(file_per_var, False)
         self.linearization = get_default(linearization, Struct(kind='strip'))
@@ -504,13 +515,18 @@ class Problem(Struct):
         conf_equations = get_default(conf_equations,
                                      self.conf.get('equations', None))
 
-        self.set_variables()
+        self.set_variables(self.conf_variables)
         variables = Variables.from_conf(self.conf_variables, self.fields)
 
-        self.set_materials()
+        self.set_materials(self.conf_materials)
         materials = Materials.from_conf(self.conf_materials, self.functions)
 
         self.integrals = self.get_integrals()
+
+        default_user = vars(self.conf)
+        if user is not None:
+            default_user.update(user)
+        user = default_user
         equations = Equations.from_conf(conf_equations, variables,
                                         self.domain.regions,
                                         materials, self.integrals,
@@ -611,10 +627,12 @@ class Problem(Struct):
         functions = get_default(functions, self.functions)
 
         ac = self.active_only
-        graph_changed = self.equations.time_update(self.ts,
-                                                   ebcs, epbcs, lcbcs,
-                                                   functions, self,
-                                                   active_only=ac)
+        graph_changed = self.equations.time_update(
+                                       self.ts,
+                                       ebcs, epbcs, lcbcs,
+                                       functions, self,
+                                       active_only=ac,
+                                       verbose=self.conf.get('verbose', True))
         self.graph_changed = graph_changed
 
         if (is_matrix
@@ -636,6 +654,10 @@ class Problem(Struct):
             conf_ebc = get_default(ebcs, self.conf.ebcs)
             self.ebcs = Conditions.from_conf(conf_ebc, self.domain.regions)
 
+            conf_dgebc = self.conf.get("dgebcs", {})
+            self.ebcs.extend(Conditions.from_conf(conf_dgebc,
+                                                  self.domain.regions))
+
         if isinstance(epbcs, Conditions):
             self.epbcs = epbcs
 
@@ -643,6 +665,10 @@ class Problem(Struct):
             conf_epbc = get_default(epbcs, self.conf.epbcs)
             self.epbcs = Conditions.from_conf(conf_epbc, self.domain.regions)
 
+            conf_dgepbc = self.conf.get("dgepbcs", {})
+            self.ebcs.extend(Conditions.from_conf(conf_dgepbc,
+                                                  self.domain.regions))
+
         if isinstance(lcbcs, Conditions):
             self.lcbcs = lcbcs
 
@@ -704,8 +730,8 @@ class Problem(Struct):
     def create_state(self):
         return State(self.equations.variables)
 
-    def get_mesh_coors(self):
-        return self.domain.get_mesh_coors()
+    def get_mesh_coors(self, actual=False):
+        return self.domain.get_mesh_coors(actual=actual)
 
     def set_mesh_coors(self, coors, update_fields=False, actual=False,
                        clear_all=True, extra_dofs=False):
@@ -762,7 +788,8 @@ class Problem(Struct):
     def init_time(self, ts):
         self.update_time_stepper(ts)
         self.equations.init_time(ts)
-        self.update_materials(mode='force')
+        self.update_materials(mode='force',
+                              verbose=self.conf.get('verbose', True))
 
         self._restart_filenames = []
 
@@ -985,23 +1012,36 @@ class Problem(Struct):
     def set_conf_solvers(self, conf_solvers=None, options=None):
         """
         Choose which solvers should be used. If solvers are not set in
-        `options`, use first suitable in `conf_solvers`.
+        `options`, use the ones named `ls`, `nls` or `ts`. If such solver names
+        do not exist, use the first of each required solver kind listed in
+        `conf_solvers`.
         """
         conf_solvers = get_default(conf_solvers, self.conf.solvers)
         self.solver_confs = {}
+
         for key, val in six.iteritems(conf_solvers):
             self.solver_confs[val.name] = val
 
         def _find_suitable(prefix):
+            cands = []
             for key, val in six.iteritems(self.solver_confs):
                 if val.kind.find(prefix) == 0:
-                    return val
-            return None
+                    if val.name == prefix[:-1]:
+                        return val
+                    else:
+                        cands.append(val)
+            if len(cands) > 0:
+                return cands[0]
+            else:
+                return None
 
         def _get_solver_conf(kind):
             try:
                 key = options[kind]
-                conf = self.solver_confs[key]
+                if key is None:
+                    conf = None
+                else:
+                    conf = self.solver_confs[key]
             except:
                 conf = _find_suitable(kind + '.')
             return conf
@@ -1050,7 +1090,18 @@ class Problem(Struct):
             nls_conf = get_default(nls_conf, self.nls_conf,
                                    'you must set nonlinear solver!')
 
-            ls = Solver.any_from_conf(ls_conf, context=self)
+            fb_list = []
+            for ii in range(100):
+                fb_list.append((ls_conf.kind, ls_conf))
+                if hasattr(ls_conf, 'fallback'):
+                    ls_conf = self.solver_confs[ls_conf.fallback]
+                else:
+                    break
+
+            if len(fb_list) > 1:
+                ls = use_first_available(fb_list, context=self)
+            else:
+                ls = Solver.any_from_conf(ls_conf, context=self)
 
             ev = self.get_evaluator()
 
@@ -1133,9 +1184,9 @@ class Problem(Struct):
     def try_presolve(self, mtx):
         ls = self.get_ls()
 
-        tt = time.clock()
+        timer = Timer(start=True)
         ls.presolve(mtx)
-        tt = time.clock() - tt
+        tt = timer.stop()
         output('presolve: %.2f [s]' % tt)
 
     def get_solver(self):
@@ -1206,7 +1257,7 @@ class Problem(Struct):
                 state.apply_ebc()
 
             if update_materials:
-                self.update_materials()
+                self.update_materials(verbose=self.conf.get('verbose', True))
 
         def poststep_fun(ts, vec):
             state = state0.copy(preserve_caches=True)
@@ -1229,7 +1280,8 @@ class Problem(Struct):
                 self.save_state(filename, state,
                                 post_process_hook=post_process_hook,
                                 file_per_var=None,
-                                ts=ts)
+                                ts=ts,
+                                file_format=self.file_format)
 
             self.advance(ts)
 
@@ -1550,7 +1602,7 @@ class Problem(Struct):
         --------
         `problem` is Problem instance.
 
-        >>> out = problem.create_evaluable('dq_state_in_volume_qp.i1.Omega(u)')
+        >>> out = problem.create_evaluable('ev_volume_integrate.i1.Omega(u)')
         >>> equations, variables = out
 
         `vec` is a vector of coefficients compatible with the field
@@ -1979,7 +2031,7 @@ class Problem(Struct):
                    ' file discards higher order DOFs! Use with caution!')
 
             fd.close()
-            io = MeshIO.for_format(filename)
+            io = MeshIO.any_from_filename(filename)
 
             out = io.read_data(step=ts.step)
 
