@@ -1,11 +1,11 @@
 from __future__ import absolute_import
-import time
 
 import numpy as nm
 import numpy.linalg as nla
 import scipy as sc
 
 from sfepy.base.base import output, get_default, dict_to_struct, assert_, Struct
+from sfepy.base.timing import Timer
 from sfepy.solvers import eig, Solver
 from sfepy.linalg import norm_l2_along_axis
 from sfepy.discrete.evaluate import eval_equations
@@ -129,7 +129,7 @@ def get_log_freqs(f0, f1, df, freq_eps, n_point_min, n_point_max):
     f_mid = 0.5 * (f0 + f1)
 
     if (f1 - f0) > (2.0 * freq_eps):
-        num = min(n_point_max, max(n_point_min, (f1 - f0) / df))
+        num = min(n_point_max, max(n_point_min, int((f1 - f0) / df)))
         a = nm.linspace(0., 1., num)
         log_freqs = f0 + freq_eps \
                     + 0.5 * (nm.sin((a - 0.5) * nm.pi) + 1.0) \
@@ -533,12 +533,13 @@ class SimpleEVP(CorrMiniApp):
         problem = get_default(problem, self.problem)
         opts = self.app_options
 
-        problem.set_equations(self.equations)
-        problem.select_bcs(ebc_names=self.ebcs, epbc_names=self.epbcs,
-                           lcbc_names=self.get('lcbcs', []))
-        problem.update_materials(problem.ts)
+        if self.equations is not None:
+            problem.set_equations(self.equations)
+            problem.select_bcs(ebc_names=self.ebcs, epbc_names=self.epbcs,
+                            lcbc_names=self.get('lcbcs', []))
+            problem.update_materials(problem.ts)
 
-        self.init_solvers(problem)
+            self.init_solvers(problem)
 
         mtx_a, mtx_m, data = self.prepare_matrices(problem)
 
@@ -637,8 +638,8 @@ class SchurEVP(SimpleEVP):
         mtx = equations.eval_tangent_matrices(None, problem.mtx_a,
                                               by_blocks=True)
 
-        ls = Solver.any_from_conf(problem.ls_conf,
-                                  presolve=True, mtx=mtx['D'])
+        ls = Solver.any_from_conf(problem.ls_conf
+                                  + Struct(use_presolve=True), mtx=mtx['D'])
 
         mtx_b, mtx_m = mtx['B'], mtx['M']
         mtx_dib = nm.empty(mtx_b.shape, dtype=mtx_b.dtype)
@@ -710,7 +711,6 @@ class DensityVolumeInfo(MiniAppBase):
             total_volume += vol
 
         true_volume = self._get_volume(volume)
-        assert_(abs(total_volume - true_volume) / true_volume < 1e-12)
 
         output('total volume:', true_volume)
 
@@ -787,11 +787,11 @@ class Eigenmomenta(MiniAppBase):
         else:
             wrap_transform = None
 
-        tt = time.clock()
+        timer = Timer(start=True)
         eigenmomenta = compute_eigenmomenta(self.expression, opts.var_name,
                                             problem, evp.eig_vectors,
                                             wrap_transform)
-        output('...done in %.2f s' % (time.clock() - tt))
+        output('...done in %.2f s' % timer.stop())
 
         n_eigs = evp.eigs.shape[0]
 
