@@ -6,19 +6,22 @@ import numpy as nm
 
 from sfepy.base.base import get_default, Struct, output
 from sfepy.base.ioutils import get_print_info
+from sfepy.base.timing import Timer
 from sfepy.discrete.fem import extend_cell_data, Mesh
 from sfepy.homogenization.utils import coor_to_sym
 from sfepy.base.conf import get_standard_keywords
 from sfepy.discrete import Problem
 from sfepy.base.conf import ProblemConf
 from sfepy.homogenization.coefficients import Coefficients
-from sfepy.homogenization.micmac import get_correctors_from_file
+from sfepy.homogenization.micmac import get_correctors_from_file_hdf5
 import os.path as op
 import six
 from six.moves import range
 
 shared = Struct()
 
+cc_cache = {}
+
 #
 # TODO : interpolate fvars to macro times. ?mid-points?
 #
@@ -275,7 +278,7 @@ def add_stress_p(out, pb, integral, region, vp, data):
 
 
 def compute_mac_stress_part(pb, integral, region, material, vu, mac_strain):
-    avgmat = pb.evaluate('ev_integrate_mat.%s.%s(%s, %s)'
+    avgmat = pb.evaluate('ev_volume_integrate_mat.%s.%s(%s, %s)'
                          % (integral, region, material, vu), verbose=False,
                          mode='el_avg')
 
@@ -480,7 +483,7 @@ def save_recovery_region(mac_pb, rname, filename=None):
 def recover_micro_hook(micro_filename, region, macro,
                        naming_scheme='step_iel',
                        recovery_file_tag='',
-                       define_args=None):
+                       define_args=None, verbose=False):
     # Create a micro-problem instance.
     required, other = get_standard_keywords()
     required.remove('equations')
@@ -493,7 +496,7 @@ def recover_micro_hook(micro_filename, region, macro,
 
     # Coefficients and correctors
     coefs = Coefficients.from_file_hdf5(coefs_filename)
-    corrs = get_correctors_from_file(dump_names=coefs.dump_names)
+    corrs = get_correctors_from_file_hdf5(dump_names=coefs.save_names)
 
     recovery_hook = conf.options.get('recovery_hook', None)
 
@@ -503,14 +506,21 @@ def recover_micro_hook(micro_filename, region, macro,
 
         format = get_print_info(pb.domain.mesh.n_el, fill='0')[1]
 
+        output('recovering microsctructures...')
+        timer = Timer(start=True)
+        output_fun = output.output_function
+        output_level = output.level
         for ii, iel in enumerate(region.cells):
-            print('ii: %d, iel: %d' % (ii, iel))
+            output.level = output_level
+            output('micro: %d (el=%d)' % (ii, iel))
 
             local_macro = {}
             for k, v in six.iteritems(macro):
                 local_macro[k] = v[ii, 0]
 
+            output.set_output(quiet=not(verbose))
             out = recovery_hook(pb, corrs, local_macro)
+            output.output_function = output_fun
 
             if ii == 0:
                 new_keys = []
@@ -535,6 +545,8 @@ def recover_micro_hook(micro_filename, region, macro,
                 pb.save_state(filename, out=out,
                               file_per_var=fpv)
 
+        output('...done in %.2f s' % timer.stop())
+
         for jj in new_keys:
             lout = new_data[jj]
             macro[jj] = nm.zeros((nm.max(new_idxs) + 1, 1) + lout[0].shape,
@@ -547,7 +559,7 @@ def recover_micro_hook(micro_filename, region, macro,
 def recover_micro_hook_eps(micro_filename, region,
                            eval_var, nodal_values, const_values, eps0,
                            recovery_file_tag='',
-                           define_args=None):
+                           define_args=None, verbose=False, use_cache=False):
     # Create a micro-problem instance.
     required, other = get_standard_keywords()
     required.remove('equations')
@@ -558,9 +570,16 @@ def recover_micro_hook_eps(micro_filename, region,
     output_dir = conf.options.get('output_dir', '.')
     coefs_filename = op.join(output_dir, coefs_filename) + '.h5'
 
+    if not use_cache:
+        cc_cache.clear()
+ 
     # Coefficients and correctors
-    coefs = Coefficients.from_file_hdf5(coefs_filename)
-    corrs = get_correctors_from_file(dump_names=coefs.dump_names)
+    if coefs_filename not in cc_cache:
+        coefs = Coefficients.from_file_hdf5(coefs_filename)
+        corrs = get_correctors_from_file_hdf5(dump_names=coefs.save_names)
+        cc_cache[coefs_filename] = coefs, corrs
+    else:
+        coefs, corrs = cc_cache[coefs_filename]
 
     recovery_hook = conf.options.get('recovery_hook', None)
 
@@ -586,31 +605,45 @@ def recover_micro_hook_eps(micro_filename, region,
             x0[:, ii] = icoor.flatten()
 
         mesh = pb.domain.mesh
+
         coors, conn, outs, ndoffset = [], [], [], 0
         # Recover region
+        mic_coors = (mesh.coors - mesh.get_bounding_box()[0, :]) * eps0
+        evfield = eval_var.field
+
+        output('recovering microsctructures...')
+        timer = Timer(start=True)
+        output_fun = output.output_function
+        output_level = output.level
+
         for ii, c0 in enumerate(x0):
             local_macro = {'eps0': eps0}
-            local_coors = pb.domain.mesh.coors * eps0 + c0
+            local_coors = mic_coors + c0
             # Inside recovery region?
-            v = nm.ones((eval_var.field.region.entities[0].shape[0], 1))
-            v[region.entities[0]] = 0
+            v = nm.ones((evfield.region.entities[0].shape[0], 1))
+            v[evfield.vertex_remap[region.entities[0]]] = 0
             no = nm.sum(v)
-            aux = eval_var.field.evaluate_at(local_coors, v)
-            if (nm.sum(aux) / no) > 1e-3:
+            aux = evfield.evaluate_at(local_coors, v)
+            if no > 0 and (nm.sum(aux) / no) > 1e-3:
                 continue
 
-            output('ii: %d' % ii)
+            output.level = output_level
+            output('micro: %d / %d' % (ii, x0.shape[0]))
 
             for k, v in six.iteritems(nodal_values):
-                local_macro[k] = eval_var.field.evaluate_at(local_coors, v)
+                local_macro[k] = evfield.evaluate_at(local_coors, v)
             for k, v in six.iteritems(const_values):
                 local_macro[k] = v
 
+            output.set_output(quiet=not(verbose))
             outs.append(recovery_hook(pb, corrs, local_macro))
+            output.output_function = output_fun
             coors.append(local_coors)
             conn.append(mesh.get_conn(mesh.descs[0]) + ndoffset)
             ndoffset += mesh.n_nod
 
+    output('...done in %.2f s' % timer.stop())
+
     # Collect output variables
     outvars = {}
     for k, v in six.iteritems(outs[0]):
