@@ -346,7 +346,7 @@ int32 dq_finite_strain( FMField *mtxF, FMField *detF, FMField *vecCS,
     // Determinant of deformation gradient.
     geme_det3x3( detF->val, mtxF );
     for (iqp = 0; iqp < nQP; iqp++) {
-      if (detF->val[iqp] <= MachEps) {
+      if (detF->val[iqp] <= 0.0) {
         errput( "warp violation %e at (iel: %d, iqp: %d)!\n",
                 detF->val[iqp], ii, iqp );
       }
@@ -444,7 +444,7 @@ int32 dq_tl_finite_strain_surface( FMField *mtxF, FMField *detF, FMField *mtxFI,
     // Determinant of deformation gradient.
     geme_det3x3( detF->val, mtxF );
     for (iqp = 0; iqp < nQP; iqp++) {
-      if (detF->val[iqp] <= MachEps) {
+      if (detF->val[iqp] <= 0.0) {
         errput( "warp violation %e at (iel: %d, iqp: %d)!\n",
                 detF->val[iqp], iel, iqp );
       }
