@@ -134,11 +134,11 @@ class SUPGCAdjStabilizationTerm(Term):
 
     def get_fargs(self, mat, virtual, state, parameter,
                   mode=None, term_mode=None, diff_var=None, **kwargs):
-        ap, vg = self.get_approximation(state)
+        vg, _ = self.get_mapping(state)
 
         val_u = self.get(parameter, 'val')
         grad_u = self.get(parameter, 'grad').transpose((0, 1, 3, 2)).copy()
-        conn = ap.get_connectivity(self.region, self.integration)
+        conn = state.field.get_connectivity(self.region, self.integration)
 
         fmode = diff_var is not None
 
@@ -169,10 +169,10 @@ class SUPGPAdj1StabilizationTerm(Term):
 
     def get_fargs(self, mat, virtual, state, parameter,
                   mode=None, term_mode=None, diff_var=None, **kwargs):
-        ap_w, vg_w = self.get_approximation(state)
+        vg_w, _ = self.get_mapping(state)
 
         grad_p = self.get(parameter, 'grad')
-        conn_w = ap_w.get_connectivity(self.region, self.integration)
+        conn_w = state.field.get_connectivity(self.region, self.integration)
 
         fmode = diff_var is not None
 
@@ -204,11 +204,11 @@ class SUPGPAdj2StabilizationTerm(Term):
 
     def get_fargs(self, mat, virtual, parameter, state,
                   mode=None, term_mode=None, diff_var=None, **kwargs):
-        ap_r, vg_r = self.get_approximation(state)
+        vg_r, _ = self.get_mapping(state)
         vg_u, _ = self.get_mapping(parameter)
 
         grad_u = self.get(parameter, 'grad').transpose((0, 1, 3, 2)).copy()
-        conn_r = ap_r.get_connectivity(self.region, self.integration)
+        conn_r = state.field.get_connectivity(self.region, self.integration)
 
         fmode = diff_var is not None
 
@@ -221,8 +221,8 @@ class SDDotVolumeTerm(Term):
     :Definition:
 
     .. math::
-        \int_{\Omega_D} p q (\nabla \cdot \ul{\Vcal}) \mbox{ , }
-        \int_{\Omega_D} (\ul{u} \cdot \ul{w}) (\nabla \cdot \ul{\Vcal})
+        \int_{\Omega} p q (\nabla \cdot \ul{\Vcal}) \mbox{ , }
+        \int_{\Omega} (\ul{u} \cdot \ul{w}) (\nabla \cdot \ul{\Vcal})
 
     :Arguments:
         - parameter_1 : :math:`p` or :math:`\ul{u}`
@@ -231,8 +231,9 @@ class SDDotVolumeTerm(Term):
     """
     name = 'd_sd_volume_dot'
     arg_types = ('parameter_1', 'parameter_2', 'parameter_mesh_velocity')
-    arg_shapes = {'parameter_1' : 'D', 'parameter_2' : 'D',
-                  'parameter_mesh_velocity' : 'D'}
+    arg_shapes = [{'parameter_1' : 'D', 'parameter_2' : 'D',
+                   'parameter_mesh_velocity' : 'D'},
+                  {'parameter_1' : 1, 'parameter_2' : 1}]
 
     function = staticmethod(terms.d_sd_volume_dot)
 
@@ -261,7 +262,7 @@ class SDDivTerm(Term):
     :Definition:
 
     .. math::
-        \int_{\Omega_D} p [ (\nabla \cdot \ul{w}) (\nabla \cdot \ul{\Vcal})
+        \int_{\Omega} p [ (\nabla \cdot \ul{w}) (\nabla \cdot \ul{\Vcal})
         - \pdiff{\Vcal_k}{x_i} \pdiff{w_i}{x_k} ]
 
     :Arguments:
@@ -304,7 +305,7 @@ class SDDivGradTerm(Term):
     :Definition:
 
     .. math::
-        w \nu \int_{\Omega_D} [ \pdiff{u_i}{x_k} \pdiff{w_i}{x_k}
+        w \nu \int_{\Omega} [ \pdiff{u_i}{x_k} \pdiff{w_i}{x_k}
         (\nabla \cdot \ul{\Vcal})
         - \pdiff{\Vcal_j}{x_k} \pdiff{u_i}{x_j} \pdiff{w_i}{x_k}
         - \pdiff{u_i}{x_k} \pdiff{\Vcal_l}{x_k} \pdiff{w_i}{x_k} ]
@@ -352,7 +353,7 @@ class SDConvectTerm(Term):
     :Definition:
 
     .. math::
-        \int_{\Omega_D} [ u_k \pdiff{u_i}{x_k} w_i (\nabla \cdot \Vcal)
+        \int_{\Omega} [ u_k \pdiff{u_i}{x_k} w_i (\nabla \cdot \Vcal)
         - u_k \pdiff{\Vcal_j}{x_k} \pdiff{u_i}{x_j} w_i ]
 
     :Arguments:
@@ -473,7 +474,7 @@ class SDGradDivStabilizationTerm(Term):
     :Definition:
 
     .. math::
-        \gamma \int_{\Omega_D} [ (\nabla \cdot \ul{u}) (\nabla \cdot \ul{w})
+        \gamma \int_{\Omega} [ (\nabla \cdot \ul{u}) (\nabla \cdot \ul{w})
         (\nabla \cdot \ul{\Vcal})
         - \pdiff{u_i}{x_k} \pdiff{\Vcal_k}{x_i} (\nabla \cdot \ul{w})
         - (\nabla \cdot \ul{u}) \pdiff{w_i}{x_k} \pdiff{\Vcal_k}{x_i} ]
