@@ -3,6 +3,7 @@ import numpy as nm
 from sfepy.base.base import assert_
 from sfepy.linalg import dot_sequences
 from sfepy.terms.terms import Term, terms
+from sfepy.terms.terms_dot import ScalarDotMGradScalarTerm
 
 class DiffusionTerm(Term):
     r"""
@@ -77,6 +78,49 @@ class DiffusionTerm(Term):
         else:
             self.function = terms.d_diffusion
 
+class SDDiffusionTerm(Term):
+    r"""
+    Diffusion sensitivity analysis term.
+
+    :Definition:
+
+    .. math::
+        \int_{\Omega} \left[ (\dvg \ul{\Vcal}) K_{ij} \nabla_i q\, \nabla_j p -
+        K_{ij} (\nabla_j \ul{\Vcal} \nabla q) \nabla_i p - K_{ij} \nabla_j q
+        (\nabla_i \ul{\Vcal} \nabla p)\right]
+
+    :Arguments:
+        - material:    :math:`K_{ij}`
+        - parameter_q: :math:`q`
+        - parameter_p: :math:`p`
+        - parameter_mesh_velocity: :math:`\ul{\Vcal}`
+    """
+    name = 'd_sd_diffusion'
+    arg_types = ('material', 'parameter_q', 'parameter_p',
+                 'parameter_mesh_velocity')
+    arg_shapes = {'material' : 'D, D',
+                  'parameter_q' : 1, 'parameter_p' : 1,
+                  'parameter_mesh_velocity' : 'D'}
+
+    function = staticmethod(terms.d_sd_diffusion)
+
+    def get_fargs(self, mat, parameter_q, parameter_p, parameter_mv,
+                  mode=None, term_mode=None, diff_var=None, **kwargs):
+        vg, _ = self.get_mapping(parameter_p)
+
+        grad_q = self.get(parameter_q, 'grad')
+        grad_p = self.get(parameter_p, 'grad')
+        grad_mv = self.get(parameter_mv, 'grad')
+        div_mv = self.get(parameter_mv, 'div')
+
+        return grad_q, grad_p, grad_mv, div_mv, mat, vg
+
+    def get_eval_shape(self, mat, parameter_q, parameter_p, parameter_mv,
+                       mode=None, term_mode=None, diff_var=None, **kwargs):
+        n_el, n_qp, dim, n_en, n_c = self.get_data_shape(parameter_q)
+
+        return (n_el, 1, 1, 1), parameter_q.dtype
+
 class LaplaceTerm(DiffusionTerm):
     r"""
     Laplace term with :math:`c` coefficient. Can be
@@ -193,12 +237,11 @@ class DiffusionCoupling(Term):
 
     def get_fargs( self, mat, virtual, state,
                    mode=None, term_mode=None, diff_var=None, **kwargs):
-        ap, vg = self.get_approximation(virtual)
+        vg, _ = self.get_mapping(virtual)
 
         if mode == 'weak':
 
-            aps, vgs = self.get_approximation(state)
-            bf = aps.get_base('v', 0, self.integral)
+            vgs, _ = self.get_mapping(state)
 
             if diff_var is None:
                 if self.mode == 'weak0':
@@ -217,7 +260,7 @@ class DiffusionCoupling(Term):
                 else:
                     fmode = 3
 
-            return val, mat, bf, vg, fmode
+            return val, mat, vgs.bf, vg, fmode
 
         elif mode == 'eval':
 
@@ -371,9 +414,9 @@ class SurfaceFluxOperatorTerm(Term):
 
     def get_fargs(self, mat, virtual, state,
                   mode=None, term_mode=None, diff_var=None, **kwargs):
-        ap, sg = self.get_approximation(state)
-        sd = ap.surface_data[self.region.name]
-        bf = ap.get_base(sd.bkey, 0, self.integral)
+        sg, _ = self.get_mapping(state)
+        sd = state.field.surface_data[self.region.name]
+        bf = state.field.get_base(sd.bkey, 0, self.integral)
 
         if mat is None:
             _, n_qp, dim, _, _ = self.get_data_shape(state)
@@ -432,7 +475,7 @@ class ConvectVGradSTerm(Term):
 
         return val_v, grad_s, vvg, svg, fmode
 
-class AdvectDivFreeTerm(Term):
+class AdvectDivFreeTerm(ScalarDotMGradScalarTerm):
     r"""
     Advection of a scalar quantity :math:`p` with the advection velocity
     :math:`\ul{y}` given as a material parameter (a known function of space and
@@ -445,44 +488,15 @@ class AdvectDivFreeTerm(Term):
     .. math::
         \int_{\Omega} \nabla \cdot (\ul{y} p) q
         = \int_{\Omega} (\underbrace{(\nabla \cdot \ul{y})}_{\equiv 0}
-        + (\ul{y}, \nabla)) p) q
+        + \ul{y} \cdot \nabla) p) q
 
     :Arguments:
         - material : :math:`\ul{y}`
         - virtual  : :math:`q`
-        - virtual  : :math:`p`
+        - state    : :math:`p`
     """
     name = 'dw_advect_div_free'
     arg_types = ('material', 'virtual', 'state')
     arg_shapes = {'material' : 'D, 1', 'virtual' : ('1', 'state'),
                   'state' : '1'}
-
-    @staticmethod
-    def function(out, mat, vg, grad, fmode):
-        bf_t = vg.bf.transpose((0, 1, 3, 2))
-
-        if fmode == 0:
-            out_qp = bf_t * dot_sequences(mat, grad, 'ATB')
-
-        else:
-            bfg = vg.bfg
-
-            out_qp = bf_t * dot_sequences(mat, bfg, 'ATB')
-
-        status = vg.integrate(out, out_qp)
-
-        return status
-
-    def get_fargs(self, mat, virtual, state,
-                  mode=None, term_mode=None, diff_var=None, **kwargs):
-        vg, _ = self.get_mapping(virtual)
-
-        if diff_var is None:
-            grad = self.get(state, 'grad')
-            fmode = 0
-
-        else:
-            grad = nm.array([0], ndmin=4, dtype=nm.float64)
-            fmode = 1
-
-        return mat, vg, grad, fmode
+    mode = 'grad_state'
