@@ -123,7 +123,7 @@ class LinearElasticIsotropicTerm(LinearElasticTerm):
                   mode=None, term_mode=None, diff_var=None, **kwargs):
         from sfepy.mechanics.matcoefs import stiffness_from_lame
 
-        mat = stiffness_from_lame(self.region.dim, lam, mu)
+        mat = stiffness_from_lame(self.region.dim, lam, mu)[:, :, 0, 0, :, :]
         return LinearElasticTerm.get_fargs(self, mat, virtual, state,
                                            mode=mode, term_mode=term_mode,
                                            diff_var=diff_var, **kwargs)
@@ -275,11 +275,11 @@ class LinearElasticETHTerm(ETHTerm):
 
 class LinearPrestressTerm(Term):
     r"""
-    Linear prestress term, with the prestress :math:`\sigma_{ij}` given in
-    the usual vector form exploiting symmetry: in 3D it has 6 components
+    Linear prestress term, with the prestress :math:`\sigma_{ij}` given either
+    in the usual vector form exploiting symmetry: in 3D it has 6 components
     with the indices ordered as :math:`[11, 22, 33, 12, 13, 23]`, in 2D it has
-    3 components with the indices ordered as :math:`[11, 22, 12]`. Can be
-    evaluated.
+    3 components with the indices ordered as :math:`[11, 22, 12]`, or in the
+    matrix (possibly non-symmetric) form. Can be evaluated.
 
     :Definition:
 
@@ -297,19 +297,31 @@ class LinearPrestressTerm(Term):
     name = 'dw_lin_prestress'
     arg_types = (('material', 'virtual'),
                  ('material', 'parameter'))
-    arg_shapes = {'material' : 'S, 1', 'virtual' : ('D', None),
-                  'parameter' : 'D'}
+    arg_shapes = [{'material' : 'S, 1', 'virtual' : ('D', None),
+                  'parameter' : 'D'},
+                  {'material' : 'D, D'}]
     modes = ('weak', 'eval')
 
     def get_fargs(self, mat, virtual,
                   mode=None, term_mode=None, diff_var=None, **kwargs):
         vg, _ = self.get_mapping(virtual)
 
+        sh = mat.shape
+        is_nonsym = sh[2] == sh[3] == vg.dim and not(vg.dim == 1)
+
+        if is_nonsym:
+            mat = mat.reshape(sh[:2] + (vg.dim**2, 1))
+
         if mode == 'weak':
             return mat, vg
 
         else:
-            strain = self.get(virtual, 'cauchy_strain')
+            if is_nonsym:
+                strain = self.get(virtual, 'grad').transpose((0,1,3,2))
+                nel, nqp, nr, nc = strain.shape
+                strain = strain.reshape((nel, nqp, nr*nc, 1))
+            else:
+                strain = self.get(virtual, 'cauchy_strain')
 
             fmode = {'eval' : 0, 'el_avg' : 1, 'qp' : 2}.get(mode, 1)
             return strain, mat, vg, fmode
@@ -731,24 +743,26 @@ def _build_wave_strain_op(vec, bf):
     out = nm.einsum('ik,cqkj->cqij', nmat, bf)
     return out
 
+from sfepy.base.compat import block
+
 def _build_cauchy_strain_op(bfg):
     dim = bfg.shape[2]
     if dim == 2:
         g1, g2 = bfg[..., 0:1, :], bfg[..., 1:2, :]
         zz = nm.zeros_like(g1)
-        out = nm.block([[g1, zz],
-                        [zz, g2],
-                        [g2, g1]])
+        out = block([[g1, zz],
+                     [zz, g2],
+                     [g2, g1]])
 
     else:
         g1, g2, g3 = bfg[..., 0:1, :], bfg[..., 1:2, :], bfg[..., 2:3, :]
         zz = nm.zeros_like(g1)
-        out = nm.block([[g1, zz, zz],
-                        [zz, g2, zz],
-                        [zz, zz, g3],
-                        [g2, g1, zz],
-                        [g3, zz, g1],
-                        [zz, g3, g2]])
+        out = block([[g1, zz, zz],
+                     [zz, g2, zz],
+                     [zz, zz, g3],
+                     [g2, g1, zz],
+                     [g3, zz, g1],
+                     [zz, g3, g2]])
 
     return out
 
@@ -785,17 +799,13 @@ class ElasticWaveTerm(Term):
 
     def get_fargs(self, mat, kappa, virtual, state,
                   mode=None, term_mode=None, diff_var=None, **kwargs):
-        from sfepy.discrete.variables import create_adof_conn
+        from sfepy.discrete.variables import create_adof_conn, expand_basis
 
         geo, _ = self.get_mapping(state)
 
-        n_fa, n_qp, dim, n_fn, n_c = self.get_data_shape(virtual)
+        n_el, n_qp, dim, n_en, n_c = self.get_data_shape(virtual)
 
-        # Expand basis for all components.
-        bf = geo.bf
-        ebf = nm.zeros(bf.shape[:2] + (dim, n_fn * dim), dtype=nm.float64)
-        for ir in range(dim):
-            ebf[..., ir, ir*n_fn:(ir+1)*n_fn] = bf[..., 0, :]
+        ebf = expand_basis(geo.bf, dim)
 
         gmat = _build_wave_strain_op(kappa, ebf)
 
@@ -858,17 +868,13 @@ class ElasticWaveCauchyTerm(Term):
 
     def get_fargs(self, mat, kappa, gvar, evar,
                   mode=None, term_mode=None, diff_var=None, **kwargs):
-        from sfepy.discrete.variables import create_adof_conn
+        from sfepy.discrete.variables import create_adof_conn, expand_basis
 
         geo, _ = self.get_mapping(evar)
 
-        n_fa, n_qp, dim, n_fn, n_c = self.get_data_shape(gvar)
+        n_el, n_qp, dim, n_en, n_c = self.get_data_shape(gvar)
 
-        # Expand basis for all components.
-        bf = geo.bf
-        ebf = nm.zeros(bf.shape[:2] + (dim, n_fn * dim), dtype=nm.float64)
-        for ir in range(dim):
-            ebf[..., ir, ir*n_fn:(ir+1)*n_fn] = bf[..., 0, :]
+        ebf = expand_basis(geo.bf, dim)
 
         gmat = _build_wave_strain_op(kappa, ebf)
         emat = _build_cauchy_strain_op(geo.bfg)
