@@ -1,9 +1,12 @@
+from __future__ import absolute_import
 import os.path as op
 
 import numpy as nm
 
 from sfepy.base.conf import transform_variables
 from sfepy.base.testing import TestCommon
+import six
+from six.moves import range
 
 variables = {
     'u'       : ('unknown field', 'f', 0),
@@ -16,7 +19,7 @@ def in_dir(adir):
 def gen_datas(meshes):
     datas = {}
 
-    for key, mesh in meshes.iteritems():
+    for key, mesh in six.iteritems(meshes):
         bbox = mesh.get_bounding_box()
         nx = bbox[1,0] - bbox[0,0]
         centre = 0.5 * bbox.sum(axis=0)
@@ -98,7 +101,7 @@ def prepare_variable(filename, n_components):
 
     u = FieldVariable('u', 'parameter', field,
                       primary_var_name='(set-to-None)')
-    u.set_from_mesh_vertices(nm.c_[tuple([data] * n_components)])
+    u.set_from_mesh_vertices(data * nm.arange(1, n_components + 1)[None, :])
 
     return u
 
@@ -257,7 +260,7 @@ class Test(TestCommon):
             term = Term.new('ev_grad(u)', integral, omega, u=u)
             term.setup()
             val1 = term.evaluate(mode='qp')
-            val1 = val1.ravel()
+            val1 = val1.transpose((0, 1, 3, 2)).ravel()
 
             val2 = u.evaluate_at(coors, mode='grad').ravel()
 
@@ -278,7 +281,7 @@ class Test(TestCommon):
                      'meshes/2d/square_unit_tri.mesh']:
             self.report(name)
 
-            u = prepare_variable(op.join(data_dir, name), n_components=1)
+            u = prepare_variable(op.join(data_dir, name), n_components=5)
 
             bbox = u.field.domain.get_mesh_bounding_box()
             coors = nm.c_[tuple([nm.linspace(ii[0] + 1e-3, ii[1] - 1e-3, 100)
@@ -287,8 +290,7 @@ class Test(TestCommon):
             grad, cells, status = u.evaluate_at(coors, mode='grad',
                                                 close_limit=0.0,
                                                 ret_status=True)
-
-            agrad = nm.dot(grad[:, 0, :], nm.ones((grad.shape[2], 1)))
+            agrad = nm.dot(grad[:, :, :], nm.ones((grad.shape[2], 1)))[..., 0]
 
             eps = 1e-5
             val0 = u.evaluate_at(coors - eps, close_limit=0.0)
@@ -305,4 +307,65 @@ class Test(TestCommon):
 
             ok = ok and _ok
 
+            for ic in range(1, u.n_components):
+                _ok = nm.allclose((ic + 1) * agrad[ii, 0], agrad[ii, ic],
+                                  rtol=0.0, atol=1e-12)
+                self.report('component %d / component 0: mean: %.2f'
+                            % (ic, (agrad[ii, ic] / agrad[ii, 0]).mean()))
+                self.report('->', _ok)
+
+                ok = ok and _ok
+
+        return ok
+
+    def test_evaluate_at(self):
+        from sfepy import data_dir
+        from sfepy.discrete.fem import Mesh
+        from sfepy.discrete import Variables
+        from sfepy.discrete.fem import FEDomain, Field
+
+        meshes = {
+            'tp' : Mesh.from_file(data_dir + '/meshes/3d/block.mesh'),
+        }
+        datas = gen_datas(meshes)
+
+        fields = {
+            'scalar_tp' : ((1,1), 'Omega', 1),
+            'vector_tp' : ((3,1), 'Omega', 1),
+        }
+
+        ok = True
+        for field_name in ['scalar_tp', 'vector_tp']:
+            d = FEDomain('d', meshes['tp'])
+            d.create_region('Omega', 'all')
+
+            f = fields[field_name]
+            field = Field.from_args('f', nm.complex128, f[0],
+                                    d.regions[f[1]],
+                                    approx_order=f[2])
+            ff = {field.name : field}
+
+            vv = Variables.from_conf(transform_variables(variables), ff)
+            u = vv['u']
+
+            bbox = d.get_mesh_bounding_box()
+            t = nm.expand_dims(nm.linspace(0, 1, 100), 1)
+            coors = nm.expand_dims(bbox[1] - bbox[0], 0) * t + bbox[0]
+
+            data_r = datas[field_name]
+            data_i = 2. / (1 + datas[field_name])
+
+            u.set_from_mesh_vertices(data_r)
+            vals_r = u.evaluate_at(coors)
+            u.set_from_mesh_vertices(data_i)
+            vals_i = u.evaluate_at(coors)
+            u.set_from_mesh_vertices(data_r + data_i * 1j)
+            vals = u.evaluate_at(coors)
+
+            _ok = nm.allclose(vals_r + vals_i * 1j, vals, rtol=0.0, atol=1e-12)
+            _ok = _ok and nm.abs(vals).sum() > 1
+            self.report('evaluating complex field %s: %s' % (field_name, _ok))
+
+            ok = ok and _ok
+
         return ok
